@@ -58,7 +58,7 @@ public:
 	template<int N> explicit BString(char const (&str)[N]);
 	// str is supposed to be a static, null-terminated array of chars
 	
-	BString(std::auto_ptr<DataChunk> chunk, size_t offset = 0);
+	BString(std::auto_ptr<DataChunk> chunk, size_t skip_front = 0, size_t skip_back = 0);
 	
 	BString(IntrusivePtr<DataChunk const> const& chunk, char const* b, char const* e);
 	
@@ -116,11 +116,11 @@ BString::BString(char const (&str)[N])
 }
 
 inline
-BString::BString(std::auto_ptr<DataChunk> chunk, size_t offset)
+BString::BString(std::auto_ptr<DataChunk> chunk, size_t skip_front, size_t skip_back)
 :	m_ptrChunk(chunk.release()),
 	m_data(
-		m_ptrChunk->getDataAddr() + offset,
-		m_ptrChunk->getDataAddr() + m_ptrChunk->getDataSize()
+		m_ptrChunk->getDataAddr() + skip_front,
+		m_ptrChunk->getDataAddr() + m_ptrChunk->getDataSize() - skip_back
 	)
 {
 }
