@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
 #endif
 
 #include "JsRuntime.h"
+#include <ace/config-lite.h>
 #include <ace/Singleton.h>
 #include <ace/Synch.h>
 #include <jsapi.h>
+#include <new> // for std::bad_alloc
 
 class JsRuntime::Holder : public JsRuntime
 {
@@ -34,6 +36,10 @@ class JsRuntime::Holder : public JsRuntime
 JsRuntime::JsRuntime()
 {
 	m_pRep = JS_NewRuntime(MEM_LIMIT);
+	if (!m_pRep) {
+		throw std::bad_alloc();
+	}
+	JS_SetGCParameter(m_pRep, JSGC_MAX_MALLOC_BYTES, GC_INTERVAL);
 }
 
 JsRuntime::~JsRuntime()
