@@ -20,6 +20,8 @@
 #ifndef PRTHREAD_H_
 #define PRTHREAD_H_
 
+#include "prtypes.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -28,6 +30,15 @@ typedef struct PRThread PRThread;
 
 PRThread* PR_GetCurrentThread();
 
+typedef void (*PRThreadPrivateDTOR)(void* priv);
+
+PRStatus PR_NewThreadPrivateIndex(
+	PRUintn* newIndex, PRThreadPrivateDTOR destructor);
+
+PRStatus PR_SetThreadPrivate(PRUintn tpdIndex, void* priv);
+
+void* PR_GetThreadPrivate(PRUintn tpdIndex);
+
 #ifdef __cplusplus
 }
 #endif
