@@ -96,7 +96,7 @@ AsyncConnector::initiate(Listener& listener, Reactor& reactor,
 	
 	vector<InetAddr> resolved_addrs(DnsResolver::resolve(addr));
 	if (resolved_addrs.empty()) {
-		handleConnFailure(Listener::FAIL_DNS, 0);
+		handleConnFailure(AsyncConnectorError::NAME_RESOLUTION_FAILED, 0);
 		return;
 	}
 	
@@ -157,7 +157,7 @@ AsyncConnector::handleTimeout(ReactorTimerId const& timer_id)
 		Error err = m_lastError;
 		markInProgressConnectionsAsUndefined();
 		if (!err.isSet()) {
-			err.set(Listener::FAIL_TIMEOUT, ETIMEDOUT);
+			err.set(AsyncConnectorError::CONNECTION_TIMEOUT, ETIMEDOUT);
 		}
 		handleConnFailure(err.code, err.errnum);
 	} else {
@@ -213,7 +213,7 @@ AsyncConnector::initiateConnection(IpRecord& ip_rec)
 	assert(ip_rec.remaining_conn_attempts > 0);
 	
 	if (!m_pReactor) {
-		m_lastError.set(Listener::FAIL_OTHER, 0);
+		m_lastError.set(AsyncConnectorError::GENERIC_ERROR, 0);
 		return false;
 	}
 	
@@ -224,7 +224,7 @@ AsyncConnector::initiateConnection(IpRecord& ip_rec)
 	if (ip_rec.peer.open(SOCK_STREAM, ip_rec.resolved_addr.get_type(), 0, 0) == -1
 	    || ip_rec.peer.enable(ACE_NONBLOCK) == -1) {
 		ip_rec.peer.close();
-		m_lastError.set(Listener::FAIL_OTHER, errno);
+		m_lastError.set(AsyncConnectorError::GENERIC_ERROR, errno);
 		return false;
 	}
 	
@@ -236,7 +236,7 @@ AsyncConnector::initiateConnection(IpRecord& ip_rec)
 		TimeDelta timeout = TimeDelta::fromMsec(NEXT_ATTEMPT_TIMEOUT);
 		ip_rec.timer_id = m_pReactor->registerTimer(handler, &timeout);
 	} catch (Reactor::Exception const&) {
-		abortConnection(ip_rec, Listener::FAIL_OTHER, 0);
+		abortConnection(ip_rec, AsyncConnectorError::GENERIC_ERROR, 0);
 		return false;
 	}
 	
@@ -244,7 +244,7 @@ AsyncConnector::initiateConnection(IpRecord& ip_rec)
 	if (ACE_OS::connect(ip_rec.peer.get_handle(), sa, ip_rec.resolved_addr.get_size()) == -1) {
 		int& errno_ref = errno;
 		if (errno_ref != EINPROGRESS && errno_ref != EWOULDBLOCK) {
-			abortConnection(ip_rec, Listener::FAIL_OTHER, errno_ref);
+			abortConnection(ip_rec, AsyncConnectorError::GENERIC_ERROR, errno_ref);
 			return false;
 		}
 	}
@@ -262,17 +262,17 @@ AsyncConnector::abortConnection(IpRecord& ip_rec, Error::Code err_code, int err_
 	bool const can_retry = (
 		ip_rec.remaining_conn_attempts > 0 &&
 		ip_rec.reachability.getConnectResult() != ServerReachability::FAILED &&
-		(err_code == Listener::FAIL_REFUSED ||
-		 err_code == Listener::FAIL_UNREACHABLE)
+		(err_code == AsyncConnectorError::CONNECTION_REFUSED ||
+		 err_code == AsyncConnectorError::DESTINATION_UNREACHABLE)
 	);
 	
 	updateReachabilityInfo(ip_rec, ServerReachability::FAILED);
-	ip_rec.peer.close();
 	
 	if (ip_rec.io_handler_id) {
 		m_pReactor->unregisterHandler(ip_rec.io_handler_id);
 		ip_rec.io_handler_id = ReactorHandlerId();
 	}
+	ip_rec.peer.close();
 	
 	if (can_retry) {
 		TimeDelta timeout(TimeDelta::fromMsec(PAUSE_BEFORE_RETRY));
@@ -347,7 +347,6 @@ AsyncConnector::clearIpRecords()
 	vector<IpRecord>::iterator it = m_ipRecords.begin();
 	vector<IpRecord>::iterator const end = m_ipRecords.end();
 	for (; it != end; ++it) {
-		it->peer.close();
 		if (m_pReactor) {
 			if (it->io_handler_id) {
 				m_pReactor->unregisterHandler(it->io_handler_id);
@@ -356,6 +355,7 @@ AsyncConnector::clearIpRecords()
 				m_pReactor->unregisterTimer(it->timer_id);
 			}
 		}
+		it->peer.close();
 	}
 	m_ipRecords.clear();
 }
@@ -375,7 +375,7 @@ AsyncConnector::handleCompletion(ACE_HANDLE handle)
 	IpRecord* ip_rec = findIpRecordByHandle(handle);
 	assert(ip_rec);
 
-	Listener::FailureCode fcode = Listener::FAIL_OTHER;
+	AsyncConnectorError::Code errcode = AsyncConnectorError::GENERIC_ERROR;
 	int err = 0;
 	int errlen = sizeof(err);
 	if (ip_rec->peer.get_option(SOL_SOCKET, SO_ERROR, &err, &errlen) == 0) {
@@ -384,22 +384,22 @@ AsyncConnector::handleCompletion(ACE_HANDLE handle)
 				onConnEstablished(*ip_rec);
 				return;
 			case ETIMEDOUT:
-				fcode = Listener::FAIL_TIMEOUT;
+				errcode = AsyncConnectorError::CONNECTION_TIMEOUT;
 				break;
 			case ECONNREFUSED:
-				fcode = Listener::FAIL_REFUSED;
+				errcode = AsyncConnectorError::CONNECTION_REFUSED;
 				break;
 			case ENETUNREACH:
 			case EHOSTUNREACH:
 			case EHOSTDOWN:
-				fcode = Listener::FAIL_UNREACHABLE;
+				errcode = AsyncConnectorError::DESTINATION_UNREACHABLE;
 				break;
 		}
 	} else {
 		err = errno;
 	}
 	
-	abortConnection(*ip_rec, fcode, err);
+	abortConnection(*ip_rec, errcode, err);
 	
 	nextConnectionAttempt();
 }
@@ -464,12 +464,12 @@ AsyncConnector::onConnEstablished(IpRecord& ip_rec)
 }
 
 void
-AsyncConnector::handleConnFailure(Listener::FailureCode fcode, int errnum)
+AsyncConnector::handleConnFailure(AsyncConnectorError::Code errcode, int)
 {
 	Listener* listener = m_observerLink.getObserver();
 	abort(); // this will detach the listener
 	if (listener) {
-		listener->onConnectionFailed(fcode, errnum);
+		listener->onConnectionFailed(errcode);
 	}
 }
 
