@@ -40,8 +40,7 @@
 #include <sigc++/sigc++.h>
 #include <memory>
 
-class ServiceContext;
-class Server;
+class AbstractServer;
 class HttpRequestMetadata;
 class AbstractRequestHandler;
 class URI;
@@ -55,7 +54,8 @@ class Client :
 public:
 	typedef IntrusivePtr<RefCountableSAP<ACE_SOCK_Stream> > SocketPtr;
 	
-	Client(ServiceContext& context, SocketPtr const& client_stream);
+	Client(SocketPtr const& client_stream,
+		AbstractServer& server, Reactor& reactor);
 	
 	virtual ~Client();
 private:
@@ -86,9 +86,13 @@ private:
 	
 	void processInputData(bool eof);
 	
-	void processResponseData(SplittableBuffer& data, bool eof, long response_id);
+	void processResponseData(
+		SplittableBuffer& data, bool eof,
+		IntrusivePtr<ResponseHandler> const& response_handler);
 	
-	bool tryDiscardResponseOutput(long response_id, uintmax_t discard_size);
+	bool tryDiscardResponseOutput(
+		ResponseHandler const& response_handler,
+		uintmax_t discard_size);
 	
 	bool isRequestInProgress() const;
 	
@@ -126,8 +130,9 @@ private:
 	
 	virtual void handleWrite(ACE_HANDLE);
 	
-	ServiceContext& m_rContext;
 	SocketPtr m_ptrClientStream;
+	AbstractServer& m_rServer;
+	Reactor& m_rReactor;
 	SplittableBuffer m_inputData;
 	SplittableBuffer m_outputData;
 	bool m_isEndOfOutput;
@@ -140,8 +145,7 @@ private:
 	Reactor::IOEvents m_eventMask;
 	int m_flags;
 	int m_isWaitingForWrite;
-	long m_responseIdCounter;
-	long m_lastResponseToProduceOutput;
+	IntrusivePtr<ResponseHandler> m_ptrLastOutputWriter;
 	StrongPtr<ACE_NULL_SYNCH, Client, Client*> m_ptrSelf;
 };
 
