@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include "GzipCompressor.h"
 #include <string>
 
-class ResponseErrorInitiator;
+class RequestStatus;
 class HttpResponseMetadata;
 class HttpHeader;
 class BString;
@@ -42,9 +42,13 @@ public:
 	
 	virtual ~CompressorResponseFilter();
 	
-	virtual void processMetadata(std::auto_ptr<HttpResponseMetadata> metadata);
+	virtual void processMetadata(
+		RequestStatus& status,
+		std::auto_ptr<HttpResponseMetadata> metadata);
 	
-	virtual void processBodyData(SplittableBuffer& data, bool eof);
+	virtual void processBodyData(
+		RequestStatus& status,
+		SplittableBuffer& data, bool eof);
 private:
 	enum { OUTPUT_LIMIT = 8192 };
 	
