@@ -27,38 +27,43 @@
 #include "NonCopyable.h"
 #include "IntrusivePtr.h"
 #include "RefCountableSAP.h"
-#include "WorkerThreadPool.h"
 #include "Reactor.h"
 #include <memory>
 
 class ACE_SOCK_Acceptor;
+class ACE_SOCK_Stream;
+class ACE_INET_Addr;
 
 class ConnAcceptor
 {
 	DECLARE_NON_COPYABLE(ConnAcceptor)
 public:
+	enum StopOnSignals { STOP_ON_SIGNALS, DONT_STOP_ON_SIGNALS };
+	enum Status { ACCEPTED, FAILED, ABORTED, STOPPED_BY_SIGNAL };
+	
 	typedef IntrusivePtr<RefCountableSAP<ACE_SOCK_Acceptor> > AcceptorPtr;
 	
-	ConnAcceptor(bool stop_on_signals = true);
+	ConnAcceptor(StopOnSignals stop_on_signals);
 	
 	~ConnAcceptor();
 	
-	WorkerThreadPool& workerPool() { return m_workerPool; }
-	
 	bool add(AcceptorPtr const& acceptor);
 	
 	void remove(AcceptorPtr const& acceptor);
 	
 	void removeAll();
 	
-	int acceptConnections();
+	Status accept(ACE_SOCK_Stream& peer, ACE_INET_Addr& peer_addr);
 	
-	void forceStop();
+	void abort();
 private:
-	class EH;
+	class EvtHandler;
+	class AcceptorReadyException;
+	
+	static Status acceptConnection(AcceptorPtr const& acceptor,
+		ACE_SOCK_Stream& peer, ACE_INET_Addr& peer_addr);
 	
 	std::auto_ptr<Reactor> m_ptrReactor;
-	WorkerThreadPool m_workerPool;
 };
 
 #endif
