@@ -38,29 +38,29 @@
 #define DBG_TRAFFIC_FROM_SCRIPT_SERVER(traf)
 #define DBG_CLIENT_CONNECTION_BEGIN()
 #define DBG_CLIENT_CONNECTION_END()
-#define DBG_REGISTER_CLIENT_REQUEST(metadata)
-#define DBG_HTTP_MESSAGE_BEGIN(metadata, type)
+#define DBG_REGISTER_CLIENT_REQUEST(metadata, request_tag)
+#define DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, type)
 #define DBG_HTTP_MESSAGE_END(type)
 #define DBG_HTTP_MESSAGE_ERROR(type)
-#define DBG_INCOMING_REQUEST_BEGIN(metadata)
+#define DBG_INCOMING_REQUEST_BEGIN(metadata, request_tag)
 #define DBG_INCOMING_REQUEST_END()
-#define DBG_INCOMING_REQUEST_ERROR();
-#define DBG_OUTGOING_REQUEST_BEGIN(metadata)
+#define DBG_INCOMING_REQUEST_ERROR()
+#define DBG_OUTGOING_REQUEST_BEGIN(metadata, request_tag)
 #define DBG_OUTGOING_REQUEST_END()
 #define DBG_OUTGOING_REQUEST_ERROR()
-#define DBG_INCOMING_RESPONSE_BEGIN(metadata)
+#define DBG_INCOMING_RESPONSE_BEGIN(metadata, request_tag)
 #define DBG_INCOMING_RESPONSE_END()
 #define DBG_INCOMING_RESPONSE_ERROR()
-#define DBG_NORMAL_OUTGOING_RESPONSE_BEGIN(metadata)
-#define DBG_FILTERED_OUTGOING_RESPONSE_BEGIN(metadata)
-#define DBG_CRAFTED_OUTGOING_RESPONSE_BEGIN(metadata)
-#define DBG_ERROR_OUTGOING_RESPONSE_BEGIN(metadata)
+#define DBG_NORMAL_OUTGOING_RESPONSE_BEGIN(metadata, request_tag)
+#define DBG_FILTERED_OUTGOING_RESPONSE_BEGIN(metadata, request_tag)
+#define DBG_CRAFTED_OUTGOING_RESPONSE_BEGIN(metadata, request_tag)
+#define DBG_ERROR_OUTGOING_RESPONSE_BEGIN(metadata, request_tag)
 #define DBG_OUTGOING_RESPONSE_END()
 #define DBG_OUTGOING_RESPONSE_ERROR()
-#define DBG_SCRIPT_REQUEST_BEGIN(metadata)
+#define DBG_SCRIPT_REQUEST_BEGIN(metadata, request_tag)
 #define DBG_SCRIPT_REQUEST_END()
 #define DBG_SCRIPT_REQUEST_ERROR()
-#define DBG_SCRIPT_RESPONSE_BEGIN(metadata)
+#define DBG_SCRIPT_RESPONSE_BEGIN(metadata, request_tag)
 #define DBG_SCRIPT_RESPONSE_END()
 #define DBG_SCRIPT_RESPONSE_ERROR()
 
@@ -105,11 +105,11 @@ Debug::clientConnectionBegin()
 #define DBG_CLIENT_CONNECTION_END() \
 Debug::clientConnectionEnd()
 
-#define DBG_REGISTER_CLIENT_REQUEST(metadata) \
-Debug::registerClientRequest(metadata)
+#define DBG_REGISTER_CLIENT_REQUEST(metadata, request_tag) \
+Debug::registerClientRequest(metadata, request_tag)
 
-#define DBG_HTTP_MESSAGE_BEGIN(metadata, type) \
-Debug::httpMessageBegin(type, metadata)
+#define DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, type) \
+Debug::httpMessageBegin(type, metadata, request_tag)
 
 #define DBG_HTTP_MESSAGE_END(type) \
 Debug::httpMessageEnd(type)
@@ -117,8 +117,8 @@ Debug::httpMessageEnd(type)
 #define DBG_HTTP_MESSAGE_ERROR(type) \
 Debug::httpMessageEnd(type, true)
 
-#define DBG_INCOMING_REQUEST_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::INCOMING_REQUEST)
+#define DBG_INCOMING_REQUEST_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::INCOMING_REQUEST)
 
 #define DBG_INCOMING_REQUEST_END() \
 DBG_HTTP_MESSAGE_END(AbstractDebugAgent::INCOMING_REQUEST)
@@ -126,8 +126,8 @@ DBG_HTTP_MESSAGE_END(AbstractDebugAgent::INCOMING_REQUEST)
 #define DBG_INCOMING_REQUEST_ERROR() \
 DBG_HTTP_MESSAGE_ERROR(AbstractDebugAgent::INCOMING_REQUEST)
 
-#define DBG_OUTGOING_REQUEST_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::OUTGOING_REQUEST)
+#define DBG_OUTGOING_REQUEST_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::OUTGOING_REQUEST)
 
 #define DBG_OUTGOING_REQUEST_END() \
 DBG_HTTP_MESSAGE_END(AbstractDebugAgent::OUTGOING_REQUEST)
@@ -135,8 +135,8 @@ DBG_HTTP_MESSAGE_END(AbstractDebugAgent::OUTGOING_REQUEST)
 #define DBG_OUTGOING_REQUEST_ERROR() \
 DBG_HTTP_MESSAGE_ERROR(AbstractDebugAgent::OUTGOING_REQUEST)
 
-#define DBG_INCOMING_RESPONSE_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::INCOMING_RESPONSE)
+#define DBG_INCOMING_RESPONSE_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::INCOMING_RESPONSE)
 
 #define DBG_INCOMING_RESPONSE_END() \
 DBG_HTTP_MESSAGE_END(AbstractDebugAgent::INCOMING_RESPONSE)
@@ -144,17 +144,17 @@ DBG_HTTP_MESSAGE_END(AbstractDebugAgent::INCOMING_RESPONSE)
 #define DBG_INCOMING_RESPONSE_ERROR() \
 DBG_HTTP_MESSAGE_ERROR(AbstractDebugAgent::INCOMING_RESPONSE)
 
-#define DBG_UNMODIFIED_OUTGOING_RESPONSE_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::UNMODIFIED_OUTGOING_RESPONSE)
+#define DBG_UNMODIFIED_OUTGOING_RESPONSE_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::UNMODIFIED_OUTGOING_RESPONSE)
 
-#define DBG_FILTERED_OUTGOING_RESPONSE_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::FILTERED_OUTGOING_RESPONSE)
+#define DBG_FILTERED_OUTGOING_RESPONSE_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::FILTERED_OUTGOING_RESPONSE)
 
-#define DBG_CRAFTED_OUTGOING_RESPONSE_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::CRAFTED_OUTGOING_RESPONSE)
+#define DBG_CRAFTED_OUTGOING_RESPONSE_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::CRAFTED_OUTGOING_RESPONSE)
 
-#define DBG_ERROR_OUTGOING_RESPONSE_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::ERROR_OUTGOING_RESPONSE)
+#define DBG_ERROR_OUTGOING_RESPONSE_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::ERROR_OUTGOING_RESPONSE)
 
 #define DBG_OUTGOING_RESPONSE_END() \
 DBG_HTTP_MESSAGE_END(AbstractDebugAgent::OUTGOING_RESPONSE)
@@ -162,8 +162,8 @@ DBG_HTTP_MESSAGE_END(AbstractDebugAgent::OUTGOING_RESPONSE)
 #define DBG_OUTGOING_RESPONSE_ERROR() \
 DBG_HTTP_MESSAGE_ERROR(AbstractDebugAgent::OUTGOING_RESPONSE)
 
-#define DBG_SCRIPT_REQUEST_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::SCRIPT_REQUEST)
+#define DBG_SCRIPT_REQUEST_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::SCRIPT_REQUEST)
 
 #define DBG_SCRIPT_REQUEST_END() \
 DBG_HTTP_MESSAGE_END(AbstractDebugAgent::SCRIPT_REQUEST)
@@ -171,8 +171,8 @@ DBG_HTTP_MESSAGE_END(AbstractDebugAgent::SCRIPT_REQUEST)
 #define DBG_SCRIPT_REQUEST_ERROR() \
 DBG_HTTP_MESSAGE_ERROR(AbstractDebugAgent::SCRIPT_REQUEST)
 
-#define DBG_SCRIPT_RESPONSE_BEGIN(metadata) \
-DBG_HTTP_MESSAGE_BEGIN(metadata, AbstractDebugAgent::SCRIPT_RESPONSE)
+#define DBG_SCRIPT_RESPONSE_BEGIN(metadata, request_tag) \
+DBG_HTTP_MESSAGE_BEGIN(metadata, request_tag, AbstractDebugAgent::SCRIPT_RESPONSE)
 
 #define DBG_SCRIPT_RESPONSE_END() \
 DBG_HTTP_MESSAGE_END(AbstractDebugAgent::SCRIPT_RESPONSE)
@@ -184,6 +184,7 @@ DBG_HTTP_MESSAGE_ERROR(AbstractDebugAgent::SCRIPT_RESPONSE)
 class HttpRequestMetadata;
 class HttpResponseMetadata;
 class SplittableBuffer;
+class RequestTag;
 
 class Debug
 {
@@ -195,13 +196,16 @@ public:
 	
 	static void clientConnectionEnd();
 	
-	static void registerClientRequest(HttpRequestMetadata& metadata);
+	static void registerClientRequest(
+		HttpRequestMetadata const& metadata, RequestTag const& req_tag);
 	
 	static void httpMessageBegin(
-		HttpMessageType type, HttpRequestMetadata const& metadata);
+		HttpMessageType type, HttpRequestMetadata const& metadata,
+		RequestTag const& req_tag);
 	
 	static void httpMessageBegin(
-		HttpMessageType type, HttpResponseMetadata const& metadata);
+		HttpMessageType type, HttpResponseMetadata const& metadata,
+		RequestTag const& req_tag);
 	
 	static void httpMessageEnd(HttpMessageType type, bool error = false);
 	
