@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 #include <config.h>
 #endif
 
-class ACE_INET_Addr;
+#include <vector>
+
 class InetAddr;
+class SymbolicInetAddr;
 
 class DnsCache
 {
@@ -36,9 +38,12 @@ private:
 public:
 	static DnsCache* instance();
 	
-	virtual void put(InetAddr const& symbolic_addr, ACE_INET_Addr const& resolved_addr) = 0;
+	virtual void put(
+		SymbolicInetAddr const& symbolic_addr,
+		std::vector<InetAddr> const& resolved_addrs) = 0;
 	
-	virtual bool get(InetAddr const& symbolic_addr, ACE_INET_Addr& resolved_addr) = 0;
+	virtual std::vector<InetAddr> get(
+		SymbolicInetAddr const& symbolic_addr) = 0;
 private:
 	enum { CAPACITY = 256 };
 	enum { TIMEOUT = 120 }; // seconds
