@@ -47,6 +47,7 @@
 #include "HtmlContentValidator.h"
 #include "BString.h"
 #include "FilterGroupTag.h"
+#include "FilterGlobalFlags.h"
 #include <string>
 #include <list>
 
@@ -154,8 +155,7 @@ void
 FilterTryList::tryHtmlFilter(
 	ResponseFilterChain& chain, HttpResponseMetadata const& metadata)
 {
-	WildcardFilterGroupTag tag;
-	if (!chain.isFlagSet(tag, HtmlContentValidator::getHtmlOrXhtmlFlag())) {
+	if (!chain.globalFlags().isSet(FilterGlobalFlags::HTML_OR_XHTML)) {
 		return;
 	}
 	
