@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,12 +32,15 @@
 #include "AbstractResponseHandler.h"
 #include "FilterTryList.h"
 #include "CraftedResponse.h"
-#include "CraftedResponseGenerator.h"
-#include "ErrorResponse.h"
+#include "ErrorCodes.h"
+#include "ErrorFactory.h"
+#include "ErrorDescriptor.h"
+#include "ImmediateResponse.h"
 #include "ReplacementImage.h"
 #include "ReplacementFlash.h"
 #include "ReplacementHtml.h"
 #include "ReplacementJs.h"
+#include "ResponseFilterChain.h"
 #include "RegexFilterDescriptor.h"
 #include "SubstitutionRequestParser.h"
 #include "State.h"
@@ -47,6 +50,7 @@
 #include "AvailableFiltersOrdered.h"
 #include "BString.h"
 #include "RequestTag.h"
+#include "ArraySize.h"
 #include <string>
 
 using namespace std;
@@ -54,37 +58,37 @@ using namespace std;
 class FilteringServer::ImageSubstitutionParser : public SubstitutionRequestParser
 {
 public:
-	ImageSubstitutionParser(bool is_head_response)
-	: SubstitutionRequestParser(is_head_response) {}
-	
-	CraftedResponseGenerator& responseGenerator() { return m_responseGenerator; }
+	std::auto_ptr<ImmediateResponse> retrieveResponse() {
+		return m_ptrResponse;
+	}
 private:
 	virtual void handleResult(
 		unsigned int width, unsigned int height,
 		URI const& url, BString const& orig_path);
 	
-	CraftedResponseGenerator m_responseGenerator;
+	std::auto_ptr<ImmediateResponse> m_ptrResponse;
 };
 
 
 class FilteringServer::FlashSubstitutionParser : public SubstitutionRequestParser
 {
 public:
-	FlashSubstitutionParser(bool is_head_response)
-	: SubstitutionRequestParser(is_head_response) {}
-	
-	CraftedResponseGenerator& responseGenerator() { return m_responseGenerator; }
+	std::auto_ptr<ImmediateResponse> retrieveResponse() {
+		return m_ptrResponse;
+	}
 private:
 	virtual void handleResult(
 		unsigned int width, unsigned int height,
 		URI const& url, BString const& orig_path);
 	
-	CraftedResponseGenerator m_responseGenerator;
+	std::auto_ptr<ImmediateResponse> m_ptrResponse;
 };
 
 
-FilteringServer::FilteringServer(ServiceContext& context, ServerTimeouts const& timeouts)
-:	Server(context, timeouts)
+FilteringServer::FilteringServer(
+	ServiceContext& context, AbstractServer& delegate)
+:	m_rContext(context),
+	m_rDelegate(delegate)
 {
 }
 
@@ -99,35 +103,52 @@ FilteringServer::submitRequest(
 	IntrusivePtr<AbstractResponseHandler> const& handler)
 {
 	IntrusivePtr<AbstractRequestHandler> res;
-	if ((res = processConnectRequest(request_metadata, request_tag, handler))) {
-		// do nothing
-	} else if ((res = processSubstitutionRequest(request_metadata, request_tag, handler))) {
-		// do nothing
-	} else if ((res = processUrlPatterns(request_metadata, request_tag, handler))) {
-		// do nothing
-	} else if ((res = processAnalyzeRequest(request_metadata, request_tag, handler))) {
-		// do nothing
-	} else {
-		URI const& uri = request_metadata->requestLine().getURI();
-		std::auto_ptr<FilterTryList> filters(new FilterTryList);
-		{
-			GlobalState::ReadAccessor global_state; // hold the lock
-			if (isFilteringEnabledForURL(*global_state, uri)) {
-				applyStandardFilters(*filters, uri);
-			}
-			if (global_state->config().isClientCompressionEnabled()) {
-				filters->append(FilterTryList::FilterConstructorPtr(
-					new FilterTryList::FilterConstructor(
-						sigc::ptr_fun(&FilterTryList::tryCompressorFilter)
-					)
-				));
-			}
-		}
-		res = Server::submitRequest(request_metadata, request_tag, handler, filters);
+	
+	typedef IntrusivePtr<AbstractRequestHandler> (FilteringServer::*ProcessFunc)(
+		ConstRequestPtr const&, RequestTag const&,
+		IntrusivePtr<AbstractResponseHandler> const&);
+	
+	static ProcessFunc const process_funcs[] = {
+		&FilteringServer::processConnectRequest,
+		&FilteringServer::processSubstitutionRequest,
+		&FilteringServer::processUrlPatterns,
+		&FilteringServer::processAnalyzeRequest,
+		&FilteringServer::processGenericRequest
+	};
+	
+	ProcessFunc const* f = process_funcs;
+	ProcessFunc const* const f_end = f + ARRAY_SIZE(process_funcs);
+	for (; f != f_end && !res; ++f) {
+		res = (this->*(*f))(request_metadata, request_tag, handler);
 	}
+	
 	return res;
 }
 
+IntrusivePtr<AbstractRequestHandler>
+FilteringServer::submitRequest(
+	ConstRequestPtr const& request_metadata,
+	RequestTag const& request_tag,
+	IntrusivePtr<AbstractResponseHandler> const& handler,
+	std::auto_ptr<ImmediateResponse> response)
+{
+	return m_rDelegate.submitRequest(
+		request_metadata, request_tag, handler, response
+	);
+}
+
+bool
+FilteringServer::isIdle() const
+{
+	return m_rDelegate.isIdle();
+}
+
+bool
+FilteringServer::isOverloaded() const
+{
+	return m_rDelegate.isOverloaded();
+}
+
 IntrusivePtr<AbstractRequestHandler>
 FilteringServer::processConnectRequest(
 	ConstRequestPtr const& request_metadata,
@@ -140,18 +161,16 @@ FilteringServer::processConnectRequest(
 			GlobalState::ReadAccessor()->urlPatterns()
 			.getSubstitutionFor(url);
 		if (subst == UrlPatterns::NO_SUBSTITUTION) {
-			return Server::submitRequest(
+			return m_rDelegate.submitRequest(
 				request_metadata, request_tag, handler
 			);
 		} else {
-			CraftedResponseGenerator rgen(
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errUrlForbidden),
-					url.toString()
-				)
-			);
-			return Server::submitRequest(
-				request_metadata, request_tag, handler, rgen
+			return m_rDelegate.submitRequest(
+				request_metadata, request_tag, handler,
+				ErrorFactory::errUrlForbidden(
+					ErrorCodes::URL_FORBIDDEN,
+					std::string(), url
+				)->retrieveResponse()
 			);
 		}
 	}
@@ -168,21 +187,22 @@ FilteringServer::processSubstitutionRequest(
 	BString const si_prefix("bf-si-");
 	BString const sf_prefix("bf-sf-");
 	URI const& uri = request_metadata->requestLine().getURI();
-	bool const is_head_request = (request_metadata->requestLine().getMethod() == BString("HEAD"));
 	
-	ImageSubstitutionParser image_parser(is_head_request);
+	ImageSubstitutionParser image_parser;
 	if (image_parser.parse(uri, si_prefix)) {
-		return Server::submitRequest(
+		request_tag->flags().set(RequestTag::REQUEST_AD_SUBST);
+		return m_rDelegate.submitRequest(
 			request_metadata, request_tag, handler,
-			image_parser.responseGenerator()
+			image_parser.retrieveResponse()
 		);
 	}
 	
-	FlashSubstitutionParser flash_parser(is_head_request);
+	FlashSubstitutionParser flash_parser;
 	if (flash_parser.parse(uri, sf_prefix)) {
-		return Server::submitRequest(
+		request_tag->flags().set(RequestTag::REQUEST_AD_SUBST);
+		return m_rDelegate.submitRequest(
 			request_metadata, request_tag, handler,
-			flash_parser.responseGenerator()
+			flash_parser.retrieveResponse()
 		);
 	}
 	
@@ -196,9 +216,7 @@ FilteringServer::processUrlPatterns(
 	IntrusivePtr<AbstractResponseHandler> const& handler)
 {
 	URI const& url = request_metadata->requestLine().getURI();
-	bool const is_head_request = request_metadata->requestLine().getMethod() == BString("HEAD");
 	UrlPatterns::Substitution subst = GlobalState::ReadAccessor()->urlPatterns().getSubstitutionFor(url);
-	CraftedResponseGenerator rgen;
 	
 	if (subst == UrlPatterns::AUTO_SUBST) {
 		BString const& path = request_metadata->requestLine().getURI().getRawPath();
@@ -210,44 +228,35 @@ FilteringServer::processUrlPatterns(
 		}
 	}
 	
+	std::auto_ptr<ImmediateResponse> response;
 	switch (subst) {
 		case UrlPatterns::FORBID_HTML: {
-			rgen = sigc::bind(
-				sigc::ptr_fun(&ErrorResponse::errUrlForbidden),
-				url.toString()
-			);
+			response = ErrorFactory::errUrlForbidden(
+				ErrorCodes::URL_FORBIDDEN,
+				std::string(), url
+			)->retrieveResponse();
 			break;
 		}
 		case UrlPatterns::EMPTY_IMAGE: {
-			rgen = sigc::bind(
-				sigc::ptr_fun(&ReplacementImage::createHttpResponse),
-				is_head_request,
+			response = ReplacementImage::createHttpResponse(
 				1, 1, // width, height
-				int(HttpStatusLine::SC_FORBIDDEN)
+				HttpStatusLine::SC_FORBIDDEN
 			);
 			break;
 		}
 		case UrlPatterns::EMPTY_FLASH: {
-			rgen = sigc::bind(
-				sigc::ptr_fun(&ReplacementFlash::createHttpResponse),
-				is_head_request,
+			response = ReplacementFlash::createHttpResponse(
 				1, 1, // width, height
-				url.toString(), ReplacementFlash::DEFAULT_FRAMERATE
+				url.toString()
 			);
 			break;
 		}
 		case UrlPatterns::EMPTY_HTML: {
-			rgen = sigc::bind(
-				sigc::ptr_fun(&ReplacementHtml::createHttpResponse),
-				is_head_request
-			);
+			response = ReplacementHtml::createHttpResponse();
 			break;
 		}
 		case UrlPatterns::EMPTY_JS: {
-			rgen = sigc::bind(
-				sigc::ptr_fun(&ReplacementJs::createHttpResponse),
-				is_head_request
-			);
+			response = ReplacementJs::createHttpResponse();
 			break;
 		}
 		default: {
@@ -255,8 +264,8 @@ FilteringServer::processUrlPatterns(
 		}
 	}
 	
-	return Server::submitRequest(
-		request_metadata, request_tag, handler, rgen
+	return m_rDelegate.submitRequest(
+		request_metadata, request_tag, handler, response
 	);
 	
 }
@@ -304,12 +313,51 @@ FilteringServer::processAnalyzeRequest(
 			));
 			request_tag->flags().set(RequestTag::REQUEST_ANALYZE);
 		}
-		res = Server::submitRequest(req, request_tag, handler, filter_try_list);
+		
+		IntrusivePtr<AbstractResponseHandler> const filter_chain(
+			new ResponseFilterChain(
+				m_rContext, req, request_tag,
+				filter_try_list, handler
+			)
+		);
+		res = m_rDelegate.submitRequest(req, request_tag, filter_chain);
 	}
 	
 	return res;
 }
 
+IntrusivePtr<AbstractRequestHandler>
+FilteringServer::processGenericRequest(
+	ConstRequestPtr const& request_metadata,
+	RequestTag const& request_tag,
+	IntrusivePtr<AbstractResponseHandler> const& handler)
+{
+	URI const& uri = request_metadata->requestLine().getURI();
+	std::auto_ptr<FilterTryList> filters(new FilterTryList);
+	
+	{
+		GlobalState::ReadAccessor global_state; // hold the lock
+		if (isFilteringEnabledForURL(*global_state, uri)) {
+			applyStandardFilters(*filters, uri);
+		}
+		if (global_state->config().isClientCompressionEnabled()) {
+			filters->append(FilterTryList::FilterConstructorPtr(
+				new FilterTryList::FilterConstructor(
+					sigc::ptr_fun(&FilterTryList::tryCompressorFilter)
+				)
+			));
+		}
+	}
+	
+	IntrusivePtr<AbstractResponseHandler> const filter_chain(
+		new ResponseFilterChain(
+			m_rContext, request_metadata,
+			request_tag, filters, handler
+		)
+	);
+	return m_rDelegate.submitRequest(request_metadata, request_tag, filter_chain);
+}
+
 void
 FilteringServer::applyStandardFilters(FilterTryList& filters, URI const& request_url)
 {
@@ -362,10 +410,8 @@ FilteringServer::ImageSubstitutionParser::handleResult(
 	unsigned int width, unsigned int height,
 	URI const& url, BString const& orig_path)
 {
-	m_responseGenerator = sigc::bind(
-		sigc::ptr_fun(&ReplacementImage::createHttpResponse),
-		isHeadResponse(), width, height,
-		int(HttpStatusLine::SC_OK)
+	m_ptrResponse = ReplacementImage::createHttpResponse(
+		width, height, HttpStatusLine::SC_OK
 	);
 }
 
@@ -376,9 +422,7 @@ FilteringServer::FlashSubstitutionParser::handleResult(
 {
 	URI orig_url(url);
 	orig_url.setAbsoluteRawPath(orig_path);
-	m_responseGenerator = sigc::bind(
-		sigc::ptr_fun(&ReplacementFlash::createHttpResponse),
-		isHeadResponse(), width, height,
-		orig_url.toString(), ReplacementFlash::DEFAULT_FRAMERATE
+	m_ptrResponse = ReplacementFlash::createHttpResponse(
+		width, height, orig_url.toString()
 	);
 }
