@@ -38,6 +38,15 @@ class GzipDecompressor : public AbstractDecompressor
 public:
 	GzipDecompressor();
 	
+	/**
+	 * \brief Allows uncompressed data as input.
+	 *
+	 * The uncompressed data will be passed through unmodified.
+	 * This method must be called just after construction or
+	 * reset().
+	 */
+	void allowUncompressed() { m_allowUncompressed = true; }
+	
 	virtual ~GzipDecompressor();
 	
 	virtual void reset();
@@ -59,7 +68,8 @@ private:
 		STATE_FHCRC,
 		STATE_FHCRC_1,
 		STATE_DATA,
-		STATE_CRC32_ISIZE
+		STATE_CRC32_ISIZE,
+		STATE_UNCOMPRESSED
 	};
 	
 	enum { MAX_OUTBUF_SIZE = 16384 };
@@ -70,11 +80,16 @@ private:
 	
 	bool loadBytes(uint8_t* where, size_t num, uint8_t const*& begin, uint8_t const* end);
 	
+	bool processHeader();
+	
+	size_t passthrough(SplittableBuffer& out, size_t const limit);
+	
 	SplittableBuffer m_bufferedInput;
 	State m_state;
 	bool m_isEOF;
 	bool m_isError;
 	bool m_isFirstDataSet;
+	bool m_allowUncompressed;
 	size_t m_stateOffset;
 	uint8_t m_header[10];
 	uint8_t m_footer[8];
