@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2004  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,27 +24,45 @@
 #include <config.h>
 #endif
 
+#include "FlagOps.h"
+#include <stddef.h>
+
 class URI;
 class BString;
+struct BStringPOD;
 
 class HeuristicScore
 {
 public:
-	virtual ~HeuristicScore() {}
-	
 	enum Status { NOT_AD, PROBABLY_NOT_AD, PROBABLY_AD, AD };
 	
-	enum UrlStatus { URL_INNOCENT, URL_MEDIOCRE, URL_SUSPICIOUS };
+	enum ExpectedType { EXPECT_ANY, EXPECT_IMAGE, EXPECT_FLASH, EXPECT_HTML };
 	
 	enum UrlRelationship { URLS_SAME_HOST, URLS_RELATED, URLS_UNRELATED };
 	
+	enum UrlFlags {
+		HAS_QUERY            = 1 << 0,
+		PROBABLY_HAS_QUERY   = 1 << 1, // like: /script.php/more/data
+		HAS_EMBEDDED_URL     = 1 << 2, // like: /script?r=http://addr.com/
+		UNEXPECTED_EXTENSION = 1 << 3,
+		GIF_EXTENSION        = 1 << 4,
+		JPEG_EXTENSION       = 1 << 5,
+		HTML_EXTENSION       = 1 << 6,
+		CGI_BIN              = 1 << 7, // like: /cgi-bin/script
+		FRONT_PAGE           = 1 << 8,
+		LOOKS_LIKE_SCRIPT    = HAS_QUERY|PROBABLY_HAS_QUERY|HAS_EMBEDDED_URL
+		                       |UNEXPECTED_EXTENSION|CGI_BIN
+	};
+	
+	virtual ~HeuristicScore() {}
+	
 	virtual int getNumericScore() const = 0;
 	
 	virtual int getNumericSizeScore() const = 0;
 	
 	virtual Status getStatus() const;
 	
-	static UrlStatus getUrlStatus(URI const& url);
+	static UrlFlags getUrlStatus(URI const& url, ExpectedType type = EXPECT_ANY);
 	
 	static UrlRelationship getUrlRelationship(URI const& url, URI const& base);
 	
@@ -53,8 +71,6 @@ public:
 	static int getSizeScore(int width, int height);
 	
 	static int getHintModifier(URI const& url);
-	
-	static bool checkFileExtension(BString const& file, BString const& ext);
 private:
 	struct SizeRecord;
 	struct SizeRecordComparator;
@@ -67,9 +83,23 @@ private:
 	
 	static void chopTopLevelDomain(char const* begin, char const*& end);
 	
+	static BString extractLastPathComponent(BString const& path);
+	
+	static void chopLastPathComponent(BString& path);
+	
+	static BString extractExtension(BString const& path_component);
+	
+	static bool extensionMatchesType(
+		BString const& extension, ExpectedType type);
+	
+	static bool findMatchInsensitive(
+		BStringPOD const* records, size_t num_records, BString const& subject);
+	
 	static SizeRecord const* findSizeRecord(int width, int height);
 	
 	static SizeRecord const m_sCommonSizes[];
 };
 
+DEFINE_FLAG_OPS(HeuristicScore::UrlFlags)
+
 #endif
