@@ -78,8 +78,7 @@ HtmlNode::detachFromParent()
 	
 	m_pPrev->m_pNext = m_pNext;
 	m_pNext->m_pPrev = m_pPrev;
-	if (unref()) {
-		m_pPrev = m_pNext = this;
-		m_pParent = 0;
-	} // otherwise this object is already destroyed
+	m_pPrev = m_pNext = this;
+	m_pParent = 0;
+	unref(); // may delete this object
 }
