@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,12 +34,14 @@
 #include "HtmlEscaper.h"
 #include "HttpFetcher.h"
 #include "RequestPtr.h"
+#include "RequestTag.h"
 #include "StrongPtr.h"
 #include "BString.h"
 #include "BStringPOD.h"
 #include "SBOutStream.h"
 #include "ErrorDescriptor.h"
 #include "ErrorCodes.h"
+#include "ErrorFactory.h"
 #include "HeuristicScore.h"
 #include "ImageHeuristicScore.h"
 #include "IframeHeuristicScore.h"
@@ -361,7 +363,8 @@ struct HtmlProcessor::ScriptFetchResult
 	ScriptFetchResult()
 	: numRedirects(0), persistentCookies(false), dynamicContent(false) {}
 	
-	IntrusivePtr<ErrorDescriptor> errorDescriptor;
+	SplittableBuffer body;
+	std::auto_ptr<ErrorDescriptor> errorDescriptor;
 	int numRedirects;
 	bool persistentCookies;
 	bool dynamicContent;
@@ -425,6 +428,7 @@ HtmlProcessor::HtmlProcessor(ServiceContext& service_context)
 	m_pScriptContext(0),
 	m_pageCleanupLevel(Config::CLEANUP_OFF),
 	m_baseURL(BString()),
+	m_requestGroupId(0),
 	m_contentType(CONTENT_HTML),
 	m_pageHint(0),
 	m_pContainingNode(&m_docFragment),
@@ -442,6 +446,7 @@ HtmlProcessor::HtmlProcessor(HtmlProcessor const& parent, ScriptContext& script_
 	m_pageCleanupLevel(parent.m_pageCleanupLevel),
 	m_ptrRequestMetadata(parent.m_ptrRequestMetadata),
 	m_baseURL(parent.m_baseURL),
+	m_requestGroupId(parent.m_requestGroupId),
 	m_jsEnv(parent.m_jsEnv),
 	m_contentType(parent.m_contentType),
 	m_pageHint(parent.m_pageHint),
@@ -465,11 +470,13 @@ HtmlProcessor::~HtmlProcessor()
 }
 
 void
-HtmlProcessor::setRelatedInfo(ConstRequestPtr const& request,
+HtmlProcessor::setRelatedInfo(
+	ConstRequestPtr const& request, RequestTag const& request_tag,
 	bool xhtml_content, bool bf_analyze)
 {	
 	m_ptrRequestMetadata = request;
 	m_baseURL = request->requestLine().getURI();
+	m_requestGroupId = request_tag->getGroupId();
 	m_jsEnv.setPageURL(m_baseURL);
 	m_contentType = (xhtml_content ? CONTENT_XHTML : CONTENT_HTML);
 	m_pageHint = HeuristicScore::getHintModifier(m_baseURL);
@@ -498,6 +505,7 @@ HtmlProcessor::reset()
 	m_ptrRequestMetadata.reset(0);
 	BString empty_str;
 	URI(empty_str).swap(m_baseURL);
+	m_requestGroupId = 0;
 	m_jsEnv.reset();
 	m_contentType = CONTENT_HTML;
 	m_ptrCurTag.reset(0);
@@ -1369,8 +1377,8 @@ HtmlProcessor::processJavaScript(HtmlNode* script, bool& nuke_adjacent_noscript)
 			return;
 		}
 		
-		fetch_res = fetchScript(
-			prepareScriptFetchRequest(script_url), script_code,
+		fetchScript(
+			prepareScriptFetchRequest(script_url), fetch_res,
 			max_body_size, max_fetch_size
 		);
 		if (fetch_res.errorDescriptor.get()) {
@@ -1381,6 +1389,7 @@ HtmlProcessor::processJavaScript(HtmlNode* script, bool& nuke_adjacent_noscript)
 			processDocFragmentNode(script);
 			return;
 		} else {
+			script_code.appendDestructive(fetch_res.body);
 			//DEBUGLOG("Script fetching complete");
 			//DEBUGLOG("EXTERNAL_SCRIPT: " << std::endl << script_code.toString());
 			stripBOM(script_code);
@@ -1457,6 +1466,7 @@ HtmlProcessor::processJavaScriptOutput(
 	//DEBUGLOG("SCRIPT OUTPUT: " << context.subprocessorOutput().toBString());
 	
 	do { // just to be able to break from it
+		/*
 		if (context.isUnrelatedScriptInvolved() &&
 		    context.isOnLoadAssigned()) {
 			BString comment("<!--\r\n[BFilter] Script removed "
@@ -1464,6 +1474,7 @@ HtmlProcessor::processJavaScriptOutput(
 			res.reset(new RemoveScriptOperation(*this, script, comment));
 			break;
 		}
+		*/
 		
 		if (context.isTooManyUnrelatedScripts()) {
 			BString comment("<!--\r\n[BFilter] Script removed "
@@ -1493,11 +1504,12 @@ HtmlProcessor::processJavaScriptOutput(
 		if (!suspect) {
 			break;
 		}
-		//DEBUGLOG("Suspect score = " << suspect->getScore().getNumericScore()
-		//	<< ", location: " << suspect->getLocation());
 		
 		Action const action = adviceAction(suspect->getScore());
 		
+		//DEBUGLOG("Suspect score = " << suspect->getScore().getNumericScore()
+		//	<< ", action = " << action << ", location: " << suspect->getLocation());
+		
 		if (action == ANALYZE) {
 			res = tryMarkScriptForAnalyzing(
 				context, *suspect, script, script_code, script_src
@@ -1505,9 +1517,18 @@ HtmlProcessor::processJavaScriptOutput(
 			if (res.get()) {
 				break;
 			}
+			if (suspect->getScore().getStatus() <= HeuristicScore::PROBABLY_NOT_AD) {
+				break;
+			}
 		}
 		
 		if (action != PASS_THROUGH) {
+			if (context.isInnerHtmlInvolved()) {
+				BString comment("<!--\r\n[BFilter] Script removed "
+					"[innerHTML ad].\r\n-->");
+				res.reset(new RemoveScriptOperation(*this, script, comment));
+				break;
+			}
 			if (action == SUBSTITUTE) {
 				script->getParent()->setAdMark(true);
 			}
@@ -2224,30 +2245,32 @@ HtmlProcessor::tagTerminatesAnchor(Iterator const& begin, Iterator const& end)
 	);
 }
 
-HtmlProcessor::ScriptFetchResult
+void
 HtmlProcessor::fetchScript(
-	RequestPtr const& request, SplittableBuffer& output,
+	RequestPtr const& request, ScriptFetchResult& res,
 	size_t max_body_size, size_t max_fetch_size)
 {
-	ScriptFetchResult res;
-	
 	{
 		URI const& uri = request->requestLine().getURI();
 		GlobalState::ReadAccessor global_state;
 		if (global_state->urlPatterns().getSubstitutionFor(uri)) {
-			return res; // as if we fetched it but it was empty
+			return; // as if we fetched it but it was empty
 		}
 		if (global_state->urlPatterns().isFilteringDisabled(uri)) {
-			res.errorDescriptor.reset(new ErrorDescriptor(int(ErrorCodes::SCRIPT_FETCH_NOFILTER)));
-			return res;
+			res.errorDescriptor = ErrorFactory::errDummyError(
+				ErrorCodes::SCRIPT_FETCH_NOFILTER
+			);
+			return;
 		}
 	}
 	
 	while (true) {
-		output.clear();
+		res.body.clear();
+		RequestTag request_tag;
+		request_tag->setGroupId(m_requestGroupId);
 		HttpFetcher fetcher(
-			m_rContext.scriptServer(), request,
-			max_body_size, max_fetch_size
+			m_rContext, m_rContext.scriptServer(),
+			request, request_tag, max_body_size, max_fetch_size
 		);
 		bool follow_redirect = false;
 		while (fetcher.getStatus() == HttpFetcher::IN_PROGRESS) {
@@ -2259,10 +2282,10 @@ HtmlProcessor::fetchScript(
 				if (metadata->statusLine().isRedirect()) {
 					res.dynamicContent = true;
 					if (++res.numRedirects > MAX_SCRIPT_FETCH_REDIRECTS) {
-						res.errorDescriptor.reset(new ErrorDescriptor(
+						res.errorDescriptor = ErrorFactory::errDummyError(
 							ErrorCodes::TOO_MANY_REDIRECTS,
 							"too many redirects"
-						));
+						);
 						break;
 					}
 					URI redirect(
@@ -2293,26 +2316,23 @@ HtmlProcessor::fetchScript(
 					fetcher.responseMetadata()->statusLine().toStream(strm);
 					string status(strm.str());
 					status.resize(status.size() - 2); // remove the training \r\n
-					res.errorDescriptor.reset(new ErrorDescriptor(
+					res.errorDescriptor = ErrorFactory::errDummyError(
 						ErrorCodes::WRONG_RESPONSE_STATUS, status
-					));
+					);
 				} else {
-					output.appendDestructive(fetcher.responseBody());
+					res.body.appendDestructive(fetcher.responseBody());
 				}
 			} else if (fetcher.getStatus() == HttpFetcher::FAILED) {
-				res.errorDescriptor = fetcher.errorDescriptor();
+				res.errorDescriptor = fetcher.retrieveErrorDescriptor();
 			} else {
 				// Reactor::handle_events() returned -1?
-				res.errorDescriptor.reset(new ErrorDescriptor(
-					ErrorCodes::UNKNOWN_ERROR,
-					"unknown error"
-				));
+				res.errorDescriptor = ErrorFactory::errDummyError(
+					ErrorCodes::UNKNOWN_ERROR, "unknown error"
+				);
 			}
 		}
 		break;
 	}
-	
-	return res;
 }
 
 RequestPtr
@@ -2337,7 +2357,6 @@ HtmlProcessor::prepareScriptFetchRequest(URI const& url)
 		request.reset(new HttpRequestMetadata(
 			HttpRequestLine(BString("GET"), url, HttpVersion::HTTP_1_1)
 		));
-		request->setClientRequestId(m_ptrRequestMetadata->getClientRequestId());
 		if (HttpHeader const* hdr = m_ptrRequestMetadata->headers().
 		    getHeaderPtr(BString("User-Agent"))) {
 			request->headers().setHeader(*hdr);
@@ -2614,8 +2633,18 @@ HtmlProcessor::SubProcessor::SubProcessor(
 	m_pOldSubProcessor(context.getSubProcessor())
 {
 	m_rContext.setSubProcessor(this);
-	m_rContext.setNestLevel(m_oldNestLevel + 1);
-	if (type == INNER_HTML) {
+	
+	if (type == DOCUMENT_WRITE) {
+		m_rContext.setNestLevel(m_oldNestLevel + 1);
+	} else {
+		assert(type == INNER_HTML);
+		/*
+		 * We don't increment script nest level in this case
+		 * (only innerHtmlLevel), because in case of INNER_HTML,
+		 * the subprocessor is called to parse the output of
+		 * an existing script, for which another subprocessor
+		 * with type == DOCUMENT_WRITE already exists.
+		 */
 		m_rContext.setInnerHtmlInvolved();
 		m_rContext.setInnerHtmlLevel(m_oldInnerHtmlLevel + 1);
 	}
