@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,15 +33,12 @@ class HttpVersion;
 class HttpMessageShaper
 {
 public:
-	static void shapeRequest(HttpRequestMetadata& request);
+	static void prepareForForwarding(HttpRequestMetadata& request);
 	
-	static void shapeResponse(
-		HttpResponseMetadata& response,
-		HttpRequestLine const& request_line);
+	static void prepareForForwarding(HttpResponseMetadata& response);
 private:
-	static void removeUnsupportedFeatures(
-		HttpHeadersCollection& headers,
-		HttpVersion const& http_vers);
+	static void removeCertainHeaders(
+		HttpHeadersCollection& headers, HttpVersion const& http_vers);
 	
 	static void normalizeReferer(HttpHeadersCollection& headers);
 };
