@@ -25,13 +25,17 @@
 
 #include "JsEnvProxy.h"
 #include "JsEnvironment.h"
+#include "BString.h"
+#include "Debug.h"
 
 JsEnvProxy::JsEnvProxy()
+:	m_pageURL(BString())
 {
 }
 
 JsEnvProxy::JsEnvProxy(JsEnvProxy const& other)
-:	m_ptrEnv(other.m_ptrEnv)
+:	m_ptrEnv(other.m_ptrEnv),
+	m_pageURL(BString())
 {
 }
 
@@ -39,17 +43,34 @@ JsEnvProxy::~JsEnvProxy()
 {
 }
 
+void
+JsEnvProxy::setPageURL(URI const& url)
+{
+	if (!m_ptrEnv) {
+		m_pageURL = url;
+	} else {
+		DEBUGLOG2("Changing page URL on existing JsEnvironment?");
+	}
+}
+
 void
 JsEnvProxy::reset()
 {
-	m_ptrEnv.reset(0);
+	if (m_ptrEnv) {
+		m_ptrEnv.reset(0);
+	} else {
+		BString empty_str;
+		URI(empty_str).swap(m_pageURL);
+	}
 }
 
 JsEnvironment*
 JsEnvProxy::access()
 {
 	if (!m_ptrEnv) {
-		m_ptrEnv.reset(new JsEnvironment);
+		m_ptrEnv.reset(new JsEnvironment(m_pageURL));
+		BString empty_str;
+		URI(empty_str).swap(m_pageURL);
 	}
 	return m_ptrEnv.get();
 }
