@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "Color.h"
 #include "SplittableBuffer.h"
 #include "DataChunk.h"
+#include "ImmediateResponse.h"
 #include "CraftedResponse.h"
 #include "Date.h"
 #include "GlobalState.h"
@@ -303,10 +304,10 @@ ReplacementFlash::ReplacementFlash(int width, int height,
 	m_binaryData.append(DataChunk::resize(buf.chunk(), buf.getDataSize()));
 }
 
-auto_ptr<CraftedResponse>
+std::auto_ptr<ImmediateResponse>
 ReplacementFlash::createHttpResponse(
-	bool is_head_response, int width, int height,
-	std::string orig_url, unsigned int frame_rate)
+	int const width, int const height,
+	std::string const& orig_url, unsigned const frame_rate)
 {
 	auto_ptr<CraftedResponse> response(new CraftedResponse(HttpStatusLine::SC_OK));
 	response->metadata().headers().setHeader(
@@ -315,14 +316,10 @@ ReplacementFlash::createHttpResponse(
 	response->metadata().headers().setHeader(
 		HttpHeader(BString("Date"), Date::formatCurrentTime())
 	);
-	if (is_head_response) {
-		response->metadata().setBodyStatus(HttpResponseMetadata::BODY_FORBIDDEN);
-	} else {
-		auto_ptr<Color> color(GlobalState::ReadAccessor()->config().getBorderColor());
-		ReplacementFlash flash(width, height, orig_url.c_str(), color.get(), frame_rate);
-		response->body().appendDestructive(flash.binaryData());
-		response->metadata().setBodyStatus(HttpResponseMetadata::BODY_SIZE_KNOWN);
-		response->metadata().setBodySize(response->body().size());
-	}
-	return response;
+	auto_ptr<Color> color(GlobalState::ReadAccessor()->config().getBorderColor());
+	ReplacementFlash flash(width, height, orig_url.c_str(), color.get(), frame_rate);
+	response->body().appendDestructive(flash.binaryData());
+	response->metadata().setBodyStatus(HttpResponseMetadata::BODY_SIZE_KNOWN);
+	response->metadata().setBodySize(response->body().size());
+	return auto_ptr<ImmediateResponse>(response);
 }
