@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 
 #include "Server.h"
 #include "ResponseFilterChain.h"
-#include "ResponseErrorInitiator.h"
 #include "SharedPtr.h"
 #include "ServerConnection.h"
 #include "HttpRequestMetadata.h"
@@ -34,10 +33,12 @@
 #include "HttpHeaderStructure.h"
 #include "HttpHeaderElement.h"
 #include "FilterTryList.h"
+#include "ImmediateResponse.h"
 #include "CraftedResponse.h"
-#include "ErrorResponse.h"
 #include "ErrorDescriptor.h"
+#include "ErrorFactory.h"
 #include "ErrorCodes.h"
+#include "LoggingResponseHandler.h"
 #include "ServerConnectionPool.h"
 #include "BString.h"
 #include "SplittableBuffer.h"
@@ -75,20 +76,18 @@ using namespace std;
 
 class Server::Request :
 	public AbstractRequestHandler,
-	public AbstractDataConsumer,
-	public ResponseErrorInitiator
+	public AbstractDataConsumer
 {
 	DECLARE_NON_COPYABLE(Request)
 public:
 	Request(ServiceContext& context, ServerWeakPtr const& server_ptr,
 		ConstRequestPtr const& request_metadata,
 		RequestTag const& request_tag,
-		IntrusivePtr<AbstractResponseHandler> const& response_handler,
-		std::auto_ptr<FilterTryList> filter_try_list);
+		IntrusivePtr<AbstractResponseHandler> const& response_handler);
 	
 	virtual ~Request();
 	
-	AbstractResponseHandler& responseHandler() { return *m_ptrFilterChain; }
+	AbstractResponseHandler& responseHandler() { return *m_ptrResponseHandler; }
 	
 	bool hasData() const { return !m_data.empty() && !m_ptrErrDesc.get(); }
 	
@@ -108,18 +107,12 @@ public:
 	
 	bool restart();
 	
-	void setErrorResponse(IntrusivePtr<ErrorDescriptor> const& edesc) {
-		m_ptrErrDesc = edesc;
-	}
-	
-	IntrusivePtr<ErrorDescriptor> const& errorDescriptor() {
+	std::auto_ptr<ErrorDescriptor> const& errorDescriptor() {
 		return m_ptrErrDesc;
 	}
 	
-	std::string const& getErrorMsg() const { return m_errorMsg; }
-	
-	int getClientRequestId() const {
-		return m_ptrOrigMetadata->getClientRequestId();
+	std::auto_ptr<ErrorDescriptor> retrieveErrorDescriptor() {
+		return m_ptrErrDesc;
 	}
 	
 	ConstRequestPtr const& getOrigMetadata() const {
@@ -153,15 +146,13 @@ public:
 	bool isGoingThroughHttpProxy() const { return m_connDestIsHttpProxy; }
 	
 	void updateRoute();
-protected:
-	virtual void raiseError(IntrusivePtr<ErrorDescriptor> const& edesc);
 	
-	virtual bool isErrorRaised() const;
+	virtual void cancel(std::auto_ptr<ErrorDescriptor> edesc);
+	
+	virtual bool isCancelled() const;
 private:
 	virtual void appendBodyData(SplittableBuffer& data, bool eof);
 	
-	virtual void cancelRequest(IntrusivePtr<ErrorDescriptor> const& edesc);
-	
 	virtual void processNewData(SplittableBuffer& data, bool eof);
 	
 	void maybeReportClientIP(Config const& conf);
@@ -173,11 +164,11 @@ private:
 	ConstRequestPtr m_ptrOrigMetadata;
 	std::auto_ptr<HttpRequestMetadata> m_ptrOutgoingMetadata;
 	RequestTag m_tag;
-	std::auto_ptr<ResponseFilterChain> m_ptrFilterChain;
+	IntrusivePtr<AbstractResponseHandler> m_ptrResponseHandler;
 	ConnectionRoute m_connRoute;
 	bool m_connDestIsHttpProxy;
-	IntrusivePtr<ErrorDescriptor> m_ptrErrDesc;
-	std::string m_errorMsg;
+	bool m_isCancelled;
+	std::auto_ptr<ErrorDescriptor> m_ptrErrDesc;
 	SplittableBuffer m_data;
 	bool m_isEOF;
 	bool m_isConsumingStarted;
@@ -194,7 +185,7 @@ public:
 		ConstRequestPtr const& response_metadata,
 		RequestTag const& request_tag,
 		IntrusivePtr<AbstractResponseHandler> const& response_handler,
-		CraftedResponseGenerator const& response_generator);
+		std::auto_ptr<ImmediateResponse> response);
 	
 	~PhantomRequest();
 	
@@ -202,8 +193,14 @@ public:
 		return *m_ptrResponseHandler;
 	}
 	
-	CraftedResponseGenerator& responseGenerator() {
-		return m_responseGenerator;
+	ImmediateResponse& response() { return *m_ptrResponse; }
+	
+	ConstRequestPtr const& getRequestPtr() const {
+		return m_ptrOrigMetadata;
+	}
+	
+	HttpRequestMetadata const& getRequestMetadata() const {
+		return *m_ptrOrigMetadata;
 	}
 	
 	HttpRequestLine const& getRequestLine() const {
@@ -214,24 +211,28 @@ public:
 		return getRequestLine().getURI();
 	}
 	
-	int getClientRequestId() const {
-		return m_ptrOrigMetadata->getClientRequestId();
-	}
-	
-	bool isHeadRequest() const {
-		return getRequestLine().getMethod() == BString("HEAD");
-	}
-	
 	RequestTag const& getTag() const { return m_tag; }
+	
+	std::auto_ptr<ErrorDescriptor> const& errorDescriptor() {
+		return m_ptrErrDesc;
+	}
+	
+	std::auto_ptr<ErrorDescriptor> retrieveErrorDescriptor() {
+		return m_ptrErrDesc;
+	}
 private:
 	virtual void appendBodyData(SplittableBuffer& data, bool eof);
 	
-	virtual void cancelRequest(IntrusivePtr<ErrorDescriptor> const& edesc);
+	virtual void cancel(std::auto_ptr<ErrorDescriptor> edesc);
+	
+	virtual bool isCancelled() const;
 	
 	ConstRequestPtr m_ptrOrigMetadata;
 	RequestTag m_tag;
 	IntrusivePtr<AbstractResponseHandler> m_ptrResponseHandler;
-	CraftedResponseGenerator m_responseGenerator;
+	std::auto_ptr<ImmediateResponse> m_ptrResponse;
+	std::auto_ptr<ErrorDescriptor> m_ptrErrDesc;
+	bool m_isCancelled;
 };
 
 
@@ -305,10 +306,9 @@ Server::submitRequest(
 	RequestTag const& request_tag,
 	IntrusivePtr<AbstractResponseHandler> const& handler)
 {
-	std::auto_ptr<FilterTryList> filter_try_list(new FilterTryList);
 	IntrusivePtr<Request> request(new Request(
 		m_rContext, m_ptrSelf, request_metadata,
-		request_tag, handler, filter_try_list
+		request_tag, handler
 	));
 	validateRequest(*request);
 	enqueueRequest(request);
@@ -321,41 +321,7 @@ Server::submitRequest(
 	ConstRequestPtr const& request_metadata,
 	RequestTag const& request_tag,
 	IntrusivePtr<AbstractResponseHandler> const& handler,
-	std::auto_ptr<FilterTryList> filter_try_list)
-{
-	IntrusivePtr<Request> request(new Request(
-		m_rContext, m_ptrSelf, request_metadata,
-		request_tag, handler, filter_try_list
-	));
-	validateRequest(*request);
-	enqueueRequest(request);
-	updatePipeline();
-	return request;
-}
-
-IntrusivePtr<AbstractRequestHandler>
-Server::submitRequest(
-	ConstRequestPtr const& request_metadata,
-	RequestTag const& request_tag,
-	IntrusivePtr<AbstractResponseHandler> const& handler,
-	IntrusivePtr<ErrorDescriptor> const& error_response)
-{
-	std::auto_ptr<FilterTryList> filter_try_list(new FilterTryList);
-	IntrusivePtr<Request> request(new Request(
-		m_rContext, m_ptrSelf, request_metadata,
-		request_tag, handler, filter_try_list
-	));
-	request->setErrorResponse(error_response);
-	enqueueRequest(request);
-	return request;
-}
-
-IntrusivePtr<AbstractRequestHandler>
-Server::submitRequest(
-	ConstRequestPtr const& request_metadata,
-	RequestTag const& request_tag,
-	IntrusivePtr<AbstractResponseHandler> const& handler,
-	CraftedResponseGenerator const& response_generator)
+	std::auto_ptr<ImmediateResponse> response_generator)
 {
 	IntrusivePtr<PhantomRequest> request(new PhantomRequest(
 		request_metadata, request_tag,
@@ -365,6 +331,20 @@ Server::submitRequest(
 	return request;
 }
 
+bool
+Server::isIdle() const
+{
+	assert(m_phantomRequestQueue.size() >= m_requestQueue.size());
+	return m_phantomRequestQueue.empty();
+}
+
+bool
+Server::isOverloaded() const
+{
+	assert(m_phantomRequestQueue.size() >= m_requestQueue.size());
+	return m_phantomRequestQueue.size() > MAX_PIPELINE_SIZE;
+}
+
 void
 Server::reset()
 {
@@ -396,7 +376,7 @@ Server::enqueueRequest(IntrusivePtr<Request> const& req)
 	m_phantomRequestQueue.push_back(IntrusivePtr<PhantomRequest>());
 	
 	RequestLog::RequestType type = RequestLog::NORMAL_REQUEST;
-	if (&m_rContext.scriptServer() == this) {
+	if (m_rContext.isScriptServer(this)) {
 		type = RequestLog::INTERNAL_REQUEST;
 	} else if (req->getTag()->flags().isSet(RequestTag::REQUEST_ANALYZE)) {
 		type = RequestLog::ANALYZE_REQUEST;
@@ -409,10 +389,11 @@ Server::enqueuePhantomRequest(IntrusivePtr<PhantomRequest> const& req)
 {
 	m_phantomRequestQueue.push_back(req);
 	
-	RequestLog::logRequest(
-		req->getTag(), req->getRequestURI(),
-		RequestLog::SUBST_REQUEST
-	);
+	RequestLog::RequestType type = RequestLog::NORMAL_REQUEST;
+	if (req->getTag()->flags().isSet(RequestTag::REQUEST_AD_SUBST)) {
+		type = RequestLog::SUBST_REQUEST;
+	}
+	RequestLog::logRequest(req->getTag(), req->getRequestURI(), type);
 }
 
 void
@@ -437,9 +418,7 @@ Server::updatePipeline()
 		
 		Request& next_req = *m_requestQueue[m_pipelineSize];
 		
-		if (next_req.errorDescriptor().get()) {
-			// This doesn't mean there won't be error requests in pipeline.
-			// They may appear there as a result of cancelRequest().
+		if (next_req.isCancelled()) {
 			break;
 		}
 		
@@ -519,16 +498,13 @@ Server::validateRequestProtocol(Request& req)
 	URI const& uri = req.getRequestURI();
 	BString const& scheme = uri.getScheme();
 	if (!StringUtils::ciEqual(scheme.begin(), scheme.end(), http.begin(), http.end())) {
-		req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
+		req.cancel(
+			ErrorFactory::errUnsupportedProtocol(
 				ErrorCodes::UNSUPPORTED_PROTOCOL,
 				"unsupported protocol",
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errUnsupportedProtocol),
-					uri.toString(), uri.getScheme().toStdString()
-				)
+				uri, uri.getScheme().toStdString()
 			)
-		));
+		);
 		return false;
 	}
 	
@@ -547,16 +523,13 @@ Server::validateRequestExpectations(Request& req)
 		list<HttpHeaderElement>::iterator const end = structure.elements().end();
 		for (; it != end; ++it) {
 			if (!InsensitiveEqual()(it->getName(), BString("100-continue"))) {
-				req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-					new ErrorDescriptor(
+				req.cancel(
+					ErrorFactory::errExpectationFailed(
 						ErrorCodes::EXPECTATION_FAILED,
-						"expectation failed",
-						sigc::bind(
-							sigc::ptr_fun(&ErrorResponse::errExpectationFailed),
-							uri.toString(), it->getName().toStdString()
-						)
+						"expectation failed", uri,
+						it->getName().toStdString()
 					)
-				));
+				);
 				return false;
 			}
 		}
@@ -575,16 +548,12 @@ Server::validateTunnelPort(Request& req)
 	URI const& uri = req.getRequestURI();
 	unsigned const port = uri.guessPort();
 	if (!GlobalState::ReadAccessor()->config().isTunnelPortAllowed(port)) {
-		req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
+		req.cancel(
+			ErrorFactory::errUrlForbidden(
 				ErrorCodes::TUNNEL_PORT_NOT_ALLOWED,
-				"tunnel port not allowed",
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errUrlForbidden),
-					uri.toString()
-				)
+				"tunnel port not allowed", uri
 			)
-		));
+		);
 		return false;
 	}
 	
@@ -716,13 +685,13 @@ Server::writeRequestData(Request& req)
 		m_writeTimer.postpone();
 		NetworkActivityReporter::instance()->reportActivity();
 #ifdef DEBUG
-		bool is_script_server = (&m_rContext.scriptServer() == this);
+		bool const is_script_server = m_rContext.isScriptServer(this);
 		if (!req.isConsumingStarted()) {
 			AbstractDebugAgent::HttpMessageType msg_type =
 				is_script_server
 				? AbstractDebugAgent::SCRIPT_REQUEST
 				: AbstractDebugAgent::OUTGOING_REQUEST;
-			DBG_HTTP_MESSAGE_BEGIN(req.getOutgoingMetadata(), msg_type);
+			DBG_HTTP_MESSAGE_BEGIN(req.getOutgoingMetadata(), req.getTag(), msg_type);
 		}
 		
 		SplittableBuffer traf;
@@ -795,12 +764,11 @@ Server::processInputData(bool eof)
 		
 		req->getTag()->downloadProgress().received() += body_data.size();
 		
-		if (!req->errorDescriptor().get()) {
+		if (!req->isCancelled()) {
 			if (status == MESSAGE_ENDS) {
-				RequestLog::logResponseEnd(req->getTag());
 #ifdef DEBUG
 				AbstractDebugAgent::HttpMessageType msg_type =
-					(&m_rContext.scriptServer() == this)
+					m_rContext.isScriptServer(this)
 					? AbstractDebugAgent::SCRIPT_RESPONSE
 					: AbstractDebugAgent::INCOMING_RESPONSE;
 				DBG_HTTP_MESSAGE_END(msg_type);
@@ -812,7 +780,9 @@ Server::processInputData(bool eof)
 				
 				// This may result in a recursive invocation
 				// of reactor event loop.
-				req->responseHandler().processBodyData(body_data, body_eof);
+				req->responseHandler().processBodyData(
+					*req, body_data, body_eof
+				);
 			}
 		}
 		
@@ -820,8 +790,8 @@ Server::processInputData(bool eof)
 			// Removed from the pipeline by recursive invocation
 			// of reactor event loop
 		} else if (req->errorDescriptor().get()) {
-			// request has been cancelled
-			processArtificialResponses();
+			assert(req->isCancelled());
+			processImmediateResponses();
 			// This will drop the server connection if neccessary.
 			// Dropping the connection also causes switchToInactive() to be called.
 		} else if (status == MESSAGE_ENDS) {
@@ -831,19 +801,20 @@ Server::processInputData(bool eof)
 				m_ptrConnection->registerResponse(m_lastResponseVersion);
 			}
 			shiftRequestQueue();
-			if (eof || !m_ptrConnection.get() || !m_ptrConnection->isExpectedToPersist()) {
+			if (eof || !m_ptrConnection.get() ||
+			    !m_ptrConnection->isExpectedToPersist()) {
 				restartPendingRequests();
 				dropConnection();
 				// m_inputData may still contain some useful data
 			}
-			processArtificialResponses();
+			processImmediateResponses();
 			updatePipeline();
 		} else if (status == ERROR_STATE) {
 			// keep in mind: an eof while in ResponseStart state generates an error
 			/*
 			DEBUGLOG("ERROR parsing response: "
 				<< (errorDescriptor().get()
-				? errorDescriptor()->getErrorMessage()
+				? errorDescriptor()->getDebugMessage()
 				: errorMessage()));
 			DEBUGLOG("empty_input = " << empty_input
 				<< ", have_connection = " << (m_ptrConnection.get() ? true : false)
@@ -878,25 +849,22 @@ Server::processInputData(bool eof)
 				*/
 #ifdef DEBUG
 				AbstractDebugAgent::HttpMessageType msg_type =
-					(&m_rContext.scriptServer() == this)
+					m_rContext.isScriptServer(this)
 					? AbstractDebugAgent::SCRIPT_RESPONSE
 					: AbstractDebugAgent::INCOMING_RESPONSE;
 				DBG_HTTP_MESSAGE_ERROR(msg_type);
 #endif
 				if (errorDescriptor().get()) {
-					answerWithError(errorDescriptor());
+					answerWithError(retrieveErrorDescriptor());
 				} else {
-					answerWithError(IntrusivePtr<ErrorDescriptor>(
-						new ErrorDescriptor(
+					answerWithError(
+						ErrorFactory::errParsingResponse(
 							ErrorCodes::ERROR_PARSING_RESPONSE,
 							"error parsing response: "+errorMessage(),
-							sigc::bind(
-								sigc::ptr_fun(&ErrorResponse::errParsingResponse),
-								m_requestQueue.front()->getRequestURI().toString(),
-								errorMessage()
-							)
+							m_requestQueue.front()->getRequestURI(),
+							errorMessage()
 						)
-					));
+					);
 				}
 			}
 			restartPendingRequests();
@@ -927,7 +895,7 @@ Server::dbgReportOutgoingRequestSent()
 {
 #ifdef DEBUG
 	AbstractDebugAgent::HttpMessageType msg_type =
-		(&m_rContext.scriptServer() == this)
+		m_rContext.isScriptServer(this)
 		? AbstractDebugAgent::SCRIPT_REQUEST
 		: AbstractDebugAgent::OUTGOING_REQUEST;
 	DBG_HTTP_MESSAGE_END(msg_type);
@@ -1021,21 +989,15 @@ Server::restartPendingRequests()
 }
 
 void
-Server::answerWithError(IntrusivePtr<ErrorDescriptor> const& edesc)
+Server::answerWithError(std::auto_ptr<ErrorDescriptor> edesc)
 {
 	assert(!m_requestQueue.empty());
 	
 	Request& req = *m_requestQueue.front();
 	
-	if (edesc->getErrorCode() == ErrorCodes::REQUEST_CANCEL_TO_SAVE_TRAFFIC) {
-		RequestLog::logResponseEnd(req.getTag());
-	} else {
-		RequestLog::logResponseError(req.getTag());
-	}
-	
 	{
 		ScopedSuspender suspender(*this);
-		req.responseHandler().processError(edesc);
+		req.responseHandler().processError(req, edesc);
 	}
 	
 	shiftRequestQueue();
@@ -1065,16 +1027,12 @@ void
 Server::onTimeout()
 {
 	URI const& uri = m_requestQueue.front()->getRequestURI();
-	answerWithError(IntrusivePtr<ErrorDescriptor>(
-		new ErrorDescriptor(
+	answerWithError(
+		ErrorFactory::errServerTimeout(
 			ErrorCodes::SERVER_TIMEOUT,
-			"server timeout",
-			sigc::bind(
-				sigc::ptr_fun(&ErrorResponse::errServerTimeout),
-				uri.toString()
-			)
+			"server timeout", uri
 		)
-	));
+	);
 	restartPendingRequests();
 	dropConnection();
 	updatePipeline();
@@ -1096,7 +1054,7 @@ Server::updateState()
 	*/
 	
 	if (!(m_flags & READING_SUSPENDED)) {
-		if (isArtificialResponseRequired()) {
+		if (isImmediateResponseRequired()) {
 			m_delayedActionTimer.ensureActive();
 		} else if (m_pipelineSize > 0 && !m_ptrConnection.get() &&
 		           !m_connector.isInProgress()) {
@@ -1161,37 +1119,19 @@ Server::processLeadingPhantomResponse()
 {
 	if (!m_phantomRequestQueue.empty()) {
 		if (PhantomRequest* req = m_phantomRequestQueue.front().get()) {
-			auto_ptr<CraftedResponse> response = req->responseGenerator()();
+			ScopedSuspender suspender(*this);
 			
-			if (req->isHeadRequest()) {
-				response->metadata().setBodyStatus(HttpMessageMetadata::BODY_FORBIDDEN);
-				response->body().clear();
-			}
+			req->response().output(
+				req->responseHandler(),
+				req->getRequestPtr(), *req, req->getTag()
+			);
 			
-			req->getTag()->flags().set(RequestTag::RESPONSE_CRAFTED);
-			response->metadata().setClientRequestId(req->getClientRequestId());
-			response->metadata().headers().setHeader(
-				BString("X-BFilter-Content"), BString("generated")
-			);
-#ifdef DEBUG
-			response->metadata().headers().setHeader(
-				BString("X-Request-URI"),
-				req->getRequestURI().toBString()
-			);
-#endif
-			{
-				ScopedSuspender suspender(*this);
-				req->responseHandler().processResponseMetadata(
-					response->retrieveMetadata()
-				);
-				
-				req->responseHandler().processBodyData(
-					response->body(), true
+			if (req->errorDescriptor().get()) {
+				req->responseHandler().processError(
+					*req, req->retrieveErrorDescriptor()
 				);
 			}
 			
-			RequestLog::logResponseBegin(req->getTag(), 200);
-			RequestLog::logResponseEnd(req->getTag());
 			m_phantomRequestQueue.pop_front();
 			return true;
 		}
@@ -1204,24 +1144,28 @@ Server::processLeadingErrorResponse()
 {
 	if (!m_requestQueue.empty()) {
 		Request& req = *m_requestQueue.front();
-		if (req.errorDescriptor().get()) {
+		if (req.isCancelled()) {
+			assert(req.errorDescriptor().get());
 			if (!req.isConsumingStarted()) {
-				answerWithError(req.errorDescriptor());
+				answerWithError(req.retrieveErrorDescriptor());
 			} else {
 #ifdef DEBUG
 				if (!req.isFullyConsumed()) {
 					AbstractDebugAgent::HttpMessageType msg_type =
-						(&m_rContext.scriptServer() == this)
+						m_rContext.isScriptServer(this)
 						? AbstractDebugAgent::SCRIPT_REQUEST
 						: AbstractDebugAgent::OUTGOING_REQUEST;
 					DBG_HTTP_MESSAGE_ERROR(msg_type);
 				}
 #endif
-				answerWithError(req.errorDescriptor());
+				answerWithError(req.retrieveErrorDescriptor());
 				restartPendingRequests();
 				dropConnection();
 				m_inputData.clear();
 			}
+			// Note: we have answerWithError() in both branches, which
+			// calls shiftRequestQueue(), so the request is removed
+			// from the queue.
 			return true;
 		}
 	}
@@ -1229,7 +1173,7 @@ Server::processLeadingErrorResponse()
 }
 
 void
-Server::processArtificialResponses()
+Server::processImmediateResponses()
 {
 	bool need_update_pipeline = false;
 	while (true) {
@@ -1247,27 +1191,28 @@ Server::processArtificialResponses()
 }
 
 bool
-Server::isArtificialResponseRequired() const
+Server::isImmediateResponseRequired() const
 {
 	if (!m_phantomRequestQueue.empty() && m_phantomRequestQueue.front().get()) {
 		return true;
 	}
-	if (!m_requestQueue.empty() && m_requestQueue.front()->errorDescriptor().get()) {
+	if (!m_requestQueue.empty() && m_requestQueue.front()->isCancelled()) {
 		return true;
 	}
 	return false;
 }
 
 void
-Server::eventGotProvisionalResponse(auto_ptr<HttpResponseMetadata> metadata)
+Server::eventGotProvisionalResponse(std::auto_ptr<HttpResponseMetadata> metadata)
 {
 	Request& req = *m_requestQueue.front();
 	ScopedSuspender suspender(*this);
-	req.responseHandler().processProvisionalResponse(metadata);
+	req.responseHandler().processProvisionalResponse(req, metadata);
 }
 
 void
-Server::eventGotMetadata(auto_ptr<HttpResponseMetadata> metadata, bool is_persistent)
+Server::eventGotMetadata(
+	std::auto_ptr<HttpResponseMetadata> metadata, bool is_persistent)
 {
 	m_lastResponseVersion = metadata->statusLine().getHttpVersion();
 	Request& req = *m_requestQueue.front();
@@ -1275,21 +1220,14 @@ Server::eventGotMetadata(auto_ptr<HttpResponseMetadata> metadata, bool is_persis
 #ifdef DEBUG
 	DEBUGLOG("Response begins: " << req.getRequestURI());
 	
-	metadata->setClientRequestId(req.getClientRequestId());
-	
 	AbstractDebugAgent::HttpMessageType msg_type =
-		(&m_rContext.scriptServer() == this)
+		m_rContext.isScriptServer(this)
 		? AbstractDebugAgent::SCRIPT_RESPONSE
 		: AbstractDebugAgent::INCOMING_RESPONSE;
-	DBG_HTTP_MESSAGE_BEGIN(*metadata, msg_type);
-	
-	metadata->headers().setHeader(
-		BString("X-Request-URI"),
-		req.getRequestURI().toBString()
-	);
+	DBG_HTTP_MESSAGE_BEGIN(*metadata, req.getTag(), msg_type);
 #endif
 	
-	HttpMessageShaper::shapeResponse(*metadata, req.getRequestLine());
+	HttpMessageShaper::prepareForForwarding(*metadata);
 	
 	if (!is_persistent && m_ptrConnection.get()) {
 		m_ptrConnection->toBeClosed();
@@ -1303,10 +1241,8 @@ Server::eventGotMetadata(auto_ptr<HttpResponseMetadata> metadata, bool is_persis
 		dp.total() = metadata->getBodySize();
 	}
 	
-	RequestLog::logResponseBegin(req.getTag(), metadata->statusLine().getCode());
-	
 	ScopedSuspender suspender(*this);
-	req.responseHandler().processResponseMetadata(metadata);
+	req.responseHandler().processResponseMetadata(req, metadata);
 }
 
 void
@@ -1330,57 +1266,35 @@ Server::onConnectionFailed(
 	
 	switch (err.getCode()) {
 		case AsyncConnectorError::NAME_RESOLUTION_FAILED: {
-			answerWithError(IntrusivePtr<ErrorDescriptor>(
-				new ErrorDescriptor(
+			answerWithError(
+				ErrorFactory::errHostNameResolution(
 					ErrorCodes::DNS_ERROR, err.toString(),
-					sigc::bind(
-						sigc::ptr_fun(&ErrorResponse::errHostNameResolution),
-						uri.toString(), addr.getHost()
-					)
+					uri, addr.getHost()
 				)
-			));
+			);
 			break;
 		}
 		case AsyncConnectorError::CONNECTION_TIMEOUT: {
-			answerWithError(IntrusivePtr<ErrorDescriptor>(
-				new ErrorDescriptor(
+			answerWithError(
+				ErrorFactory::errConnectionTimeout(
 					ErrorCodes::CONNECT_TIMEOUT, err.toString(),
-					sigc::bind(
-						sigc::ptr_fun(&ErrorResponse::errConnectionTimeout),
-						uri.toString(), addr.getHost(), addr.getPort()
-					)
+					uri, addr.getHost(), addr.getPort()
 				)
-			));
+			);
 			break;
 		}
 		case AsyncConnectorError::CONNECTION_REFUSED:
-		case AsyncConnectorError::DESTINATION_UNREACHABLE: {
-			answerWithError(IntrusivePtr<ErrorDescriptor>(
-				new ErrorDescriptor(
-					ErrorCodes::CONNECT_FAILED,
-					"connection failed",
-					sigc::bind(
-						sigc::ptr_fun(&ErrorResponse::errConnectionFailed),
-						uri.toString(), addr.getHost(),
-						addr.getPort(), err.toString()
-					)
-				)
-			));
-			break;
-		}
+		case AsyncConnectorError::DESTINATION_UNREACHABLE:
 		case AsyncConnectorError::GENERIC_ERROR:
 		default: {
-			answerWithError(IntrusivePtr<ErrorDescriptor>(
-				new ErrorDescriptor(
+			answerWithError(
+				ErrorFactory::errConnectionFailed(
 					ErrorCodes::CONNECT_FAILED,
-					"connection failed",
-					sigc::bind(
-						sigc::ptr_fun(&ErrorResponse::errConnectionFailed),
-						uri.toString(), addr.getHost(),
-						addr.getPort(), err.toString()
-					)
+					"connection failed", uri,
+					addr.getHost(), addr.getPort(),
+					err.toString()
 				)
-			));
+			);
 			break;
 		}
 	}
@@ -1412,17 +1326,13 @@ Server::handleForwardingProblem(
 	Request const& req = *m_requestQueue.front();
 	URI const& uri = req.getRequestURI();
 	
-	answerWithError(IntrusivePtr<ErrorDescriptor>(
-		new ErrorDescriptor(
-			ErrorCodes::ERROR_FORWARDING_THROUGH_PROXY,
-			problem,
-			sigc::bind(
-				sigc::ptr_fun(&ErrorResponse::errForwardingThroughProxy),
-				uri.toString(), problem, proxy.getTypeAsString(),
-				proxy.getAddr().getHost(), proxy.getAddr().getPort()
-			)
+	answerWithError(
+		ErrorFactory::errForwardingThroughProxy(
+			ErrorCodes::ERROR_FORWARDING_THROUGH_PROXY, problem,
+			uri, problem, proxy.getTypeAsString(),
+			proxy.getAddr().getHost(), proxy.getAddr().getPort()
 		)
-	));
+	);
 	
 	updatePipeline();
 }
@@ -1436,7 +1346,7 @@ Server::handleRead(ACE_HANDLE)
 		return;
 	}
 	
-	processArtificialResponses(); // it may drop the connection
+	processImmediateResponses(); // it may drop the connection
 	if (!m_ptrConnection.get()) {
 		return;
 	}
@@ -1452,7 +1362,7 @@ Server::handleRead(ACE_HANDLE)
 #ifdef DEBUG
 		SplittableBuffer traf;
 		traf.append(chunk);
-		if (this == &m_rContext.scriptServer()) {
+		if (m_rContext.isScriptServer(this)) {
 			DBG_TRAFFIC_FROM_SCRIPT_SERVER(traf);
 		} else {
 			DBG_TRAFFIC_FROM_SERVER(traf);
@@ -1517,17 +1427,13 @@ Server::processNewHttpConnection(std::auto_ptr<ServerConnection> conn)
 		string problem("Could not register a socket in the reactor: "+string(e.what()));
 		DEBUGLOG2(problem);
 		m_ptrConnection.reset(0);
-		answerWithError(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
-				ErrorCodes::UNKNOWN_ERROR,
-				problem,
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errGenericError),
-					req.getRequestURI().toString(), problem,
-					int(HttpStatusLine::SC_INTERNAL_SERVER_ERROR)
-				)
+		answerWithError(
+			ErrorFactory::errGenericError(
+				ErrorCodes::UNKNOWN_ERROR, problem,
+				req.getRequestURI(), problem,
+				HttpStatusLine::SC_INTERNAL_SERVER_ERROR
 			)
-		));
+		);
 		return;
 	}
 	
@@ -1573,12 +1479,12 @@ Server::onDelayedAction()
 		// will be handled by processArtificialResponses() below.
 	}
 	
-	if (isArtificialResponseRequired()) {
-		processArtificialResponses();
+	if (isImmediateResponseRequired()) {
+		processImmediateResponses();
 		// This may result in a recursive main loop invocation.
 	}
 	
-	assert(!isArtificialResponseRequired());
+	assert(!isImmediateResponseRequired());
 	
 	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !m_connector.isInProgress()) {
 		assert(m_pipelineSize == 1);
@@ -1596,19 +1502,18 @@ Server::Request::Request(
 	ServiceContext& context, ServerWeakPtr const& server_ptr,
 	ConstRequestPtr const& request_metadata,
 	RequestTag const& request_tag,
-	IntrusivePtr<AbstractResponseHandler> const& response_handler,
-	auto_ptr<FilterTryList> filter_try_list)
+	IntrusivePtr<AbstractResponseHandler> const& response_handler)
 :	m_rContext(context),
 	m_ptrServer(server_ptr),
 	m_ptrOrigMetadata(request_metadata),
 	m_ptrOutgoingMetadata(new HttpRequestMetadata(*request_metadata)),
 	m_tag(request_tag),
-	m_ptrFilterChain(new ResponseFilterChain(
-		context, m_ptrOrigMetadata, request_tag,
-		filter_try_list, response_handler, *this
-	)),
+	m_ptrResponseHandler(
+		new LoggingResponseHandler(response_handler, request_tag)
+	),
 	m_connRoute(SymbolicInetAddr(std::string(), -1)),
 	m_connDestIsHttpProxy(false),
+	m_isCancelled(false),
 	m_isEOF(false),
 	m_isConsumingStarted(false),
 	m_isConstructionDone(false),
@@ -1653,16 +1558,12 @@ Server::Request::updateRoute(Forwarding::Resolver const& resolver)
 		proxy_chain = resolver.resolve(uri);
 	} catch (Forwarding::ResolverException const& e) {
 		std::string const problem(e.what());
-		setErrorResponse(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
+		cancel(
+			ErrorFactory::errRemoteProxyConfig(
 				ErrorCodes::REMOTE_PROXY_CONFIG_PROBLEM,
-				problem,
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errRemoteProxyConfig),
-					uri.toString(), problem
-				)
+				problem, uri, problem
 			)
-		));
+		);
 		return;
 	}
 	
@@ -1684,46 +1585,40 @@ Server::Request::updateRoute(Forwarding::Resolver const& resolver)
 }
 
 void
-Server::Request::raiseError(IntrusivePtr<ErrorDescriptor> const& edesc)
+Server::Request::cancel(std::auto_ptr<ErrorDescriptor> edesc)
 {
-	m_ptrErrDesc = edesc;
+	if (!m_isCancelled) {
+		m_ptrErrDesc = edesc;
+		m_isCancelled = true;
+	}
 }
 
 bool
-Server::Request::isErrorRaised() const
+Server::Request::isCancelled() const
 {
-	return m_ptrErrDesc.get();
+	return m_isCancelled;
 }
 
 void
 Server::Request::appendBodyData(SplittableBuffer& data, bool eof)
 {
-	if (errorDescriptor().get()) {
+	if (m_isCancelled) {
 		return;
 	}
 	
 	m_streamWriter.appendBodyData(data, eof);
 	if (m_streamWriter.getBufferedBodyDataSize() > MAX_BUFFERED_REQUEST_BODY) {
-		string problem("request body too big");
-		cancelRequest(IntrusivePtr<ErrorDescriptor>(
-			new ErrorDescriptor(
+		string const problem("request body too big");
+		cancel(
+			ErrorFactory::errParsingRequest(
 				ErrorCodes::REQUEST_BODY_TOO_BIG, problem,
-				sigc::bind(
-					sigc::ptr_fun(&ErrorResponse::errParsingRequest),
-					getRequestURI().toString(), problem
-				)
+				getRequestURI(), problem
 			)
-		));
+		);
 		return;
 	}
 }
 
-void
-Server::Request::cancelRequest(IntrusivePtr<ErrorDescriptor> const& edesc)
-{
-	setErrorResponse(edesc);
-}
-
 void
 Server::Request::processNewData(SplittableBuffer& data, bool eof)
 {
@@ -1761,6 +1656,9 @@ Server::Request::restart()
 		return false;
 	}
 	m_data.clear();
+	m_ptrErrDesc.reset();
+	m_isEOF = false;
+	m_isCancelled = false;
 	m_isConsumingStarted = false;
 	updateRoute(); // user settings might have changed
 	
@@ -1808,11 +1706,14 @@ Server::PhantomRequest::PhantomRequest(
 	ConstRequestPtr const& request_metadata,
 	RequestTag const& request_tag,
 	IntrusivePtr<AbstractResponseHandler> const& response_handler,
-	CraftedResponseGenerator const& response_generator)
+	std::auto_ptr<ImmediateResponse> response)
 :	m_ptrOrigMetadata(request_metadata),
 	m_tag(request_tag),
-	m_ptrResponseHandler(response_handler),
-	m_responseGenerator(response_generator)
+	m_ptrResponseHandler(
+		new LoggingResponseHandler(response_handler, request_tag)
+	),
+	m_ptrResponse(response),
+	m_isCancelled(false)
 {
 }
 
@@ -1821,11 +1722,21 @@ Server::PhantomRequest::~PhantomRequest()
 }
 
 void
-Server::PhantomRequest::appendBodyData(SplittableBuffer& data, bool eof)
+Server::PhantomRequest::appendBodyData(SplittableBuffer&, bool)
 {
 }
 
 void
-Server::PhantomRequest::cancelRequest(IntrusivePtr<ErrorDescriptor> const& edesc)
+Server::PhantomRequest::cancel(std::auto_ptr<ErrorDescriptor> edesc)
 {
+	if (!m_isCancelled) {
+		m_ptrErrDesc = edesc;
+		m_isCancelled = true;
+	}
+}
+
+bool
+Server::PhantomRequest::isCancelled() const
+{
+	return m_isCancelled;
 }
