@@ -48,7 +48,8 @@ TextContentValidator::~TextContentValidator()
 }
 
 void
-TextContentValidator::processMetadata(auto_ptr<HttpResponseMetadata> metadata)
+TextContentValidator::processMetadata(
+	RequestStatus&, std::auto_ptr<HttpResponseMetadata> metadata)
 {
 	// Content-Type is checked in FilterTryList::tryTextContentValidator
 	m_ptrMetadata = metadata;
@@ -62,10 +63,11 @@ Additionally, the presence of null characters indicates binary content.
 */
 
 void
-TextContentValidator::processBodyData(SplittableBuffer& data, bool eof)
+TextContentValidator::processBodyData(
+	RequestStatus& status, SplittableBuffer& data, bool eof)
 {
 	if (!m_ptrMetadata.get()) {
-		outputBodyData(data, eof);
+		outputBodyData(status, data, eof);
 		return;
 	}
 	
@@ -94,8 +96,8 @@ TextContentValidator::processBodyData(SplittableBuffer& data, bool eof)
 			m_ptrMetadata->headers().removeHeader(BString("Content-Type"));
 			getRequestTag()->flags().set(RequestTag::RESPONSE_MODIFIED);
 		}
-		outputMetadata(m_ptrMetadata);
-		outputBodyData(m_bufferedData, eof);
+		outputMetadata(status, m_ptrMetadata);
+		outputBodyData(status, m_bufferedData, eof);
 	}
 }
 
