@@ -107,12 +107,11 @@ Timer::ref()
 	++m_refCounter;
 }
 
-bool
+void
 Timer::unref()
 {
 	if (--m_refCounter == 0) {
 		m_pReactor = 0;
 		m_id = ReactorTimerId();
 	}
-	return true;
 }
