@@ -41,6 +41,7 @@ public:
 	
 	~Timer();
 	
+	// Note: reactor and timer must not be used by different threads
 	void connectToReactor(Reactor& reactor);
 	
 	void disconnectFromReactor();
@@ -61,7 +62,7 @@ private:
 	
 	virtual void ref();
 	
-	virtual bool unref();
+	virtual void unref();
 	
 	Reactor* m_pReactor;
 	int m_refCounter; // must come before m_id
