@@ -27,6 +27,20 @@
 #include "URI.h"
 #include "BString.h"
 
+CombinedUrlPatterns&
+CombinedUrlPatterns::operator=(CombinedUrlPatterns const& other)
+{
+	CombinedUrlPatterns(other).swap(*this);
+	return *this;
+}
+
+void
+CombinedUrlPatterns::swap(CombinedUrlPatterns& other)
+{
+	m_standardPatterns.swap(other.m_standardPatterns);
+	m_localPatterns.swap(other.m_localPatterns);
+}
+
 bool
 CombinedUrlPatterns::isFilteringDisabled(URI const& url) const
 {
