@@ -35,7 +35,9 @@ public:
 	
 	~CombinedUrlPatterns() {}
 	
-	// member-wise copying is OK
+	CombinedUrlPatterns& operator=(CombinedUrlPatterns const& other);
+	
+	void swap(CombinedUrlPatterns& other);
 	
 	UrlPatterns& standardPatterns() { return m_standardPatterns; }
 	
@@ -55,4 +57,10 @@ private:
 	UrlPatterns m_localPatterns;
 };
 
+
+inline void swap(CombinedUrlPatterns& o1, CombinedUrlPatterns& o2)
+{
+	o1.swap(o2);
+}
+
 #endif
