@@ -29,6 +29,7 @@
 #include "TrayMenu.h"
 #include "AboutDialog.h"
 #include "LogDialog.h"
+#include "TwoPhaseLogRecord.h"
 #include "BasicConfigDialog.h"
 #include "AdvancedConfigWindow.h"
 #include "FilterConfigWindow.h"
@@ -43,6 +44,7 @@
 #include "WorkerThreadPoolExSingleton.h"
 #include "ScopedIncDec.h"
 #include "FileOps.h"
+#include "cache/ObjectStorage.h"
 #ifdef DEBUG
 #include "DebugWindow.h"
 #endif
@@ -103,6 +105,7 @@ Application::Application(int argc, char* argv[])
 	
 	m_userConfDir = Glib::build_filename(Glib::get_home_dir(), ".bfilter");
 	m_globalConfDir = string(SYSCONFDIR) + "/bfilter";
+	m_userCacheDir = Glib::build_filename(m_userConfDir, "cache");
 	
 	m_spInstance = this;
 	
@@ -156,6 +159,7 @@ Application::run()
 	m_ptrUrlsFile->loadAndForceApply();
 	m_ptrUrlsLocalFile->loadAndForceApply();
 	m_ptrContentFilters->loadAndApply();
+	loadCache();
 	if (errors) {
 		showLogDialog();
 	}
@@ -298,6 +302,33 @@ Application::processCommands()
 	m_dispatcher.emit(); // there could be more commands in the queue
 }
 
+bool
+Application::loadCache()
+{
+	
+	Log* log = OperationLog::instance();
+	TwoPhaseLogRecord log_record(log,
+		StyledText("Loading cache ... ", log->getDefaultStyle()),
+		StyledText("done", log->getSuccessStyle())
+	);
+	
+	if (GlobalState::ReadAccessor()->config().getCacheSize() == 0) {
+		log_record.setSecondPhaseMessage(
+			StyledText("disabled", log->getDefaultStyle())
+		);
+		return true;
+	}
+	
+	if (!HttpCache::ObjectStorage::instance()->setCacheDir(m_userCacheDir)) {
+		log_record.setSecondPhaseMessage(
+			StyledText("failed", log->getErrorStyle())
+		);
+		return false;
+	}
+	
+	return true;
+}
+
 bool
 Application::createMissingUserConfigs()
 {
@@ -316,14 +347,16 @@ Application::createMissingUserConfigs()
 	std::string const filters_dirname(
 		Glib::build_filename(m_userConfDir, "filters")
 	);
-	bool const confdir_exists = fileExists(m_userConfDir);
+	bool const conf_dir_exists = fileExists(m_userConfDir);
+	bool const cache_dir_exists = fileExists(m_userCacheDir);
 	bool const config_exists = fileExists(config_fname);
 	bool const user_forwarding_xml_exists = fileExists(user_forwarding_xml_fname);
 	bool const global_forwarding_xml_exists = fileExists(global_forwarding_xml_fname);
 	bool const urls_local_exists = fileExists(urls_local_fname);
 	bool const filters_dir_exists = fileExists(filters_dirname);
-	if (confdir_exists && config_exists && urls_local_exists && filters_dir_exists
-	    && (user_forwarding_xml_exists || !global_forwarding_xml_exists)) {
+	if (conf_dir_exists && cache_dir_exists && config_exists &&
+	    urls_local_exists && filters_dir_exists &&
+	    (user_forwarding_xml_exists || !global_forwarding_xml_exists)) {
 		return true;
 	}
 	
@@ -332,9 +365,12 @@ Application::createMissingUserConfigs()
 	size_t num_records = log->getNumRecords();
 	bool errors = false;
 	
-	if (!confdir_exists) {
+	if (!conf_dir_exists) {
 		errors |= !FileOps::createDir(m_userConfDir);
 	}
+	if (!cache_dir_exists) {
+		errors |= !FileOps::createDir(m_userCacheDir);
+	}
 	if (!config_exists) {
 		string const source(Glib::build_filename(m_globalConfDir, "config"));
 		errors |= !copyFile(source, config_fname);
