@@ -126,6 +126,8 @@ private:
 	
 	void processCommands();
 	
+	bool loadCache();
+	
 	bool createMissingUserConfigs();
 	
 	static bool copyFile(std::string const& source, std::string const& target);
@@ -146,6 +148,7 @@ private:
 	RequestLogHandler m_requestLogHandler;
 	std::string m_userConfDir;
 	std::string m_globalConfDir;
+	std::string m_userCacheDir;
 	std::auto_ptr<ConfigFile> m_ptrConfigFile;
 	std::auto_ptr<ForwardingConfigFile> m_ptrFwdConfigFile;
 	std::auto_ptr<UrlsFile> m_ptrUrlsFile;
