@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@
 #include <gdkmm/color.h>
 #include <gdkmm/gc.h>
 
+namespace GtkGUI
+{
+
 class ColorRectangle : public Gtk::DrawingArea
 {
 public:
@@ -44,4 +47,6 @@ private:
 	Gdk::Color m_color;
 };
 
+} // namespace GtkGUI
+
 #endif
