@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 
 class BString;
 class SplittableBuffer;
+
+namespace GtkGUI
+{
+
 class DebugConnection;
 
 class DebugLayout : public Gtk::Layout
@@ -134,4 +138,6 @@ private:
 	ConnectionQueue m_orderedConnections;
 };
 
+} // namespace GtkGUI
+
 #endif
