@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,6 +42,9 @@
 
 class SplittableBuffer;
 
+namespace GtkGUI
+{
+
 class DebugWindow : public Gtk::Window
 {
 public:
@@ -105,4 +108,6 @@ private:
 	AutoScrollingWindow m_requestListScrollWnd;
 };
 
+} // namespace GtkGUI
+
 #endif
