@@ -121,9 +121,9 @@ public:
 
 	void startProxyEditing(int proxy_idx);
 
-	void stopProxyOperation();
+	void stopProxyOperation(Forwarding::Option const& opt);
 
-	void commitProxyOperation();
+	void commitProxyOperation(Forwarding::Option const& opt);
 
 	void enterBypassEditingMode();
 
@@ -775,29 +775,19 @@ ForwardingConfigWindow::Impl::startProxyEditing(int const proxy_idx)
 }
 
 void
-ForwardingConfigWindow::Impl::stopProxyOperation()
+ForwardingConfigWindow::Impl::stopProxyOperation(Forwarding::Option const& opt)
 {
 	m_pProxyEditPanel->cancelCurrentOperation();
-	showEditor(SHOW_BYPASS_EDITOR);
+	showEditor(opt.proxyChain().empty() ? SHOW_NONE : SHOW_BYPASS_EDITOR);
 	m_pOptionsList->set_sensitive(true);
 	m_pProxyChainPanel->switchToNormalMode();
 }
 
 void
-ForwardingConfigWindow::Impl::commitProxyOperation()
+ForwardingConfigWindow::Impl::commitProxyOperation(Forwarding::Option const& opt)
 {
-	stopProxyOperation();
-
-	long const option_idx = m_pOptionsList->getSelectedItem();
-	if (option_idx == -1) {
-		return;
-	}
-	assert(option_idx >= 0);
-	assert(option_idx < (int)m_forwardingConfig.options().size());
-
-	Forwarding::OptionList::iterator opt(m_forwardingConfig.options().begin());
-	std::advance(opt, option_idx);
-	m_pProxyChainPanel->loadOption(*opt);
+	stopProxyOperation(opt);
+	m_pProxyChainPanel->loadOption(opt);
 }
 
 /**
@@ -2113,13 +2103,21 @@ ForwardingConfigWindow::ProxyEditPanel::onOK()
 		m_rOwner.forwardingConfig().swap(new_config);
 	}
 	
-	m_rOwner.commitProxyOperation();
+	m_rOwner.commitProxyOperation(*opt);
 }
 
 void
 ForwardingConfigWindow::ProxyEditPanel::onCancel()
 {
-	m_rOwner.stopProxyOperation();
+	Forwarding::Config const& config = m_rOwner.forwardingConfig();
+	
+	assert(m_optionIdx >= 0);
+	assert(m_optionIdx < (int)config.options().size());
+
+	Forwarding::OptionList::const_iterator opt(config.options().begin());
+	std::advance(opt, m_optionIdx);
+	
+	m_rOwner.stopProxyOperation(*opt);
 }
 
 
