@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,12 +26,18 @@
 
 #include "NonCopyable.h"
 #include <glibmm/ustring.h>
+#include <glibmm/refptr.h>
 #include <gtkmm/textbuffer.h>
 #include <gtkmm/texttag.h>
 #include <stddef.h>
 
 class LogObserver;
 
+namespace GtkGUI
+{
+
+class StyledText;
+
 class Log
 {
 	DECLARE_NON_COPYABLE(Log)
@@ -42,14 +48,18 @@ public:
 
 	~Log();
 	
+	void appendRecord(GtkGUI::StyledText const& text);
+	
 	void appendRecord(Glib::ustring const& text);
 	
 	void appendRecord(Glib::ustring const& text, StylePtr const& style);
 	
+	void appendToLastRecord(GtkGUI::StyledText const& text);
+	
 	void appendToLastRecord(Glib::ustring const& text);
-
+	
 	void appendToLastRecord(Glib::ustring const& text, StylePtr const& style);
-
+	
 	void clear();
 	
 	bool hasUnseenWarnings() const { return m_haveUnseenWarnings; }
@@ -100,17 +110,6 @@ private:
 	StylePtr m_ptrErrorStyle;
 };
 
-
-inline void
-Log::appendRecord(Glib::ustring const& text)
-{
-	appendRecord(text, m_ptrDefaultStyle);
-}
-
-inline void
-Log::appendToLastRecord(Glib::ustring const& text)
-{
-	appendToLastRecord(text, m_ptrDefaultStyle);
-}
+} // namespace GtkGUI
 
 #endif
