@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,9 @@ namespace Gtk {
 	class Adjustment;
 }
 
+namespace GtkGUI
+{
+
 class LogDialog : public Gtk::Window, private AbstractLogView
 {
 private:
@@ -49,4 +52,6 @@ private:
 	static LogDialog* m_spInstance;
 };
 
+} // namespace GtkGUI
+
 #endif
