@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,11 +29,15 @@
 #include "Log.h"
 #include "OperationLog.h"
 #include "ArraySize.h"
+#include "WorkerThreadPoolExSingleton.h"
 #include <gtkmm/window.h>
 #include <glibmm/main.h>
 
 using namespace std;
 
+namespace GtkGUI
+{
+
 TrayIcon::TrayIcon()
 {
 	mouseClickSignal().connect(
@@ -49,7 +53,7 @@ TrayIcon::TrayIcon()
 		sigc::mem_fun(*this, &TrayIcon::onUnseenWarnings)
 	);
 	
-	Application::instance()->acceptingConnectionsSignal().connect(
+	WorkerThreadPoolExSingleton::instance()->acceptingConnectionsSignal().connect(
 		sigc::mem_fun(*this, &TrayIcon::onAcceptingConnections)
 	);
 	
@@ -174,7 +178,7 @@ TrayIcon::onUnseenWarnings(bool warnings)
 void
 TrayIcon::onStateChange()
 {
-	if (!Application::instance()->isAcceptingConnections()) {
+	if (!WorkerThreadPoolExSingleton::instance()->isAcceptingConnections()) {
 		setInoperativeIcon();
 		return;
 	}
@@ -194,3 +198,5 @@ TrayIcon::onStateChange()
 		}
 	}
 }
+
+} // namespace GtkGUI
