@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,6 +30,6 @@
 int main(int argc, char *argv[])
 {
 	ACE_Sig_Action sigpipe_action(SIG_IGN, SIGPIPE);
-	Application::entryPoint(argc, argv);
+	GtkGUI::Application::entryPoint(argc, argv);
 	return 0;
 }
