@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,8 +28,6 @@
 #include <ace/config-lite.h>
 #include <wx/frame.h>
 
-class Application;
-class LogWidget;
 class wxString;
 class wxPanel;
 class wxNotebook;
@@ -38,6 +36,12 @@ class wxNotebookEvent;
 class wxActivateEvent;
 class wxCloseEvent;
 
+namespace wxGUI
+{
+
+class Application;
+class LogWidget;
+
 class AdvancedConfigWindow : public wxFrame, private AbstractLogView
 {
 private:
@@ -101,4 +105,6 @@ private:
 	DECLARE_EVENT_TABLE()
 };
 
+} // namespace wxGUI
+
 #endif
