@@ -30,6 +30,8 @@
 #include "FilterJsLogHandler.h"
 #include "RequestLogHandler.h"
 #include "types.h"
+#include <ace/config-lite.h>
+#include <ace/Mutex.h>
 #include <wx/string.h>
 #include <wx/app.h>
 #include <wx/filename.h>
@@ -120,7 +122,11 @@ private:
 	void onIdle(wxIdleEvent& evt);
 	
 	void processCommands();
+
+	bool loadCache();
 	
+	ACE_Mutex m_mainCacheMutex;
+	bool m_isMainCacheOwner;
 	bool m_isExiting;
 	int m_commandNestLevel;
 	volatile int32_t m_havePendingCommands;
@@ -130,6 +136,7 @@ private:
 	RequestLogHandler m_requestLogHandler;
 	wxFileName m_appDir;
 	wxFileName m_confDir;
+	wxFileName m_mainCacheDir;
 	std::auto_ptr<ConfigFile> m_ptrConfigFile;
 	std::auto_ptr<ForwardingConfigFile> m_ptrFwdConfigFile;
 	std::auto_ptr<UrlsFile> m_ptrUrlsFile;
