@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,8 +30,6 @@
 #include <memory>
 
 class FilterGroupTag;
-class LogWidget;
-class ContentFilterGroup;
 class wxMenuBar;
 class wxCommandEvent;
 class wxActivateEvent;
@@ -40,6 +38,11 @@ class wxTreeEvent;
 class wxTreeItemId;
 class wxString;
 
+namespace wxGUI
+{
+
+class LogWidget;
+
 class FilterConfigWindow : public wxFrame, private AbstractLogView
 {
 public:
@@ -53,8 +56,9 @@ public:
 private:
 	struct EditState;
 	class FilterTree;
-	struct FileNode;
-	struct FilterNode;
+	class TreeItemData;
+	class FileNode;
+	class FilterNode;
 	class FilterPanel;
 	friend class FilterConfigWindow::FilterTree;
 	friend class FilterConfigWindow::FilterPanel;
@@ -93,13 +97,6 @@ private:
 	
 	void markCurrentFilterAsModified();
 	
-	static bool renameFilterGroupFile(
-		ContentFilterGroup const& group, wxString const& new_name);
-	
-	static bool saveFilterGroup(
-		ContentFilterGroup const& new_group,
-		ContentFilterGroup const* old_group = 0);
-	
 	static FilterConfigWindow* m_spInstance;
 	FilterTree* m_pFilterTree;
 	FilterPanel* m_pFilterPanel;
@@ -108,4 +105,6 @@ private:
 	DECLARE_EVENT_TABLE()
 };
 
+} // namespace wxGUI
+
 #endif
