@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,9 @@
 
 using namespace std;
 
+namespace wxGUI
+{
+
 BEGIN_EVENT_TABLE(LogDialog, wxFrame)
 	EVT_ACTIVATE(onWindowActivate)
 	EVT_CLOSE(onWindowClose)
@@ -124,3 +127,5 @@ LogDialog::onClearLog(wxCommandEvent& evt)
 {
 	OperationLog::instance()->clear();
 }
+
+} // namespace wxGUI
