@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,11 +28,15 @@
 #include <ace/config-lite.h>
 #include <wx/frame.h>
 
-class LogWidget;
 class wxActivateEvent;
 class wxCloseEvent;
 class wxCommandEvent;
 
+namespace wxGUI
+{
+
+class LogWidget;
+
 class LogDialog : public wxFrame, private AbstractLogView
 {
 public:
@@ -56,4 +60,6 @@ private:
 	DECLARE_EVENT_TABLE()
 };
 
+} // namespace wxGUI
+
 #endif
