@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@
 #include <config.h>
 #endif
 
+namespace wxGUI
+{
+
 class StyledText;
 
 class LogObserver
@@ -36,4 +39,6 @@ public:
 	virtual void clear() = 0;
 };
 
+} // namespace wxGUI
+
 #endif
