@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,9 @@
 #include <ace/config-lite.h>
 #include <wx/textctrl.h>
 
+namespace wxGUI
+{
+
 class Log;
 class StyledText;
 
@@ -56,4 +59,6 @@ private:
 	Log* m_pLog;
 };
 
+} // namespace wxGUI
+
 #endif
