@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,6 +30,9 @@
 #include <wx/string.h>
 #include <wx/textctrl.h>
 
+namespace wxGUI
+{
+
 class StyledText
 {
 public:
@@ -58,4 +61,6 @@ StyledText::StyledText(
 {
 }
 
+} // namespace wxGUI
+
 #endif
