@@ -22,6 +22,7 @@
 #include "CFUtils.h"
 #include "StringLess.h"
 #include "GlobalState.h"
+#include "Forwarding.h"
 #include "ProxyDescriptor.h"
 #include "SymbolicInetAddr.h"
 #include "AssertException.h"
@@ -98,11 +99,11 @@ try
 		
 	updateProxies();
 } catch (GenericError const& e) {
-	setGenericError(e.getMessage());
+	setUnmaskableError(e.getMessage());
 } catch (AssertException const& e) {
-	setGenericError(e.getMessage());
+	setUnmaskableError(e.getMessage());
 } catch (std::exception const& e) {
-	setGenericError(string("Exception: ")+e.what());
+	setUnmaskableError(string("Exception: ")+e.what());
 }
 
 ProxyWatcher::~ProxyWatcher()
@@ -147,11 +148,11 @@ try {
 	
 	getProxiesFromPersistentStore(service_path);
 } catch (GenericError const& e) {
-	setGenericError(e.getMessage());
+	setUnmaskableError(e.getMessage());
 } catch (AssertException const& e) {
-	setGenericError(e.getMessage());
+	setUnmaskableError(e.getMessage());
 } catch (std::exception const& e) {
-	setGenericError(string("Exception: ")+e.what());
+	setUnmaskableError(string("Exception: ")+e.what());
 }
 
 void
@@ -215,7 +216,7 @@ ProxyWatcher::getProxiesFromPersistentStore(StringRef const& primary_service)
 		throw AssertException(__FILE__, __LINE__);
 	}
 	
-	getProxiesFromService(service);
+	getProxiesFromService(service, set_name);
 }
 
 StringRef
@@ -354,7 +355,7 @@ ProxyWatcher::findServiceBySetAndDeviceName(
 }
 
 void
-ProxyWatcher::getProxiesFromService(DictRef const& service)
+ProxyWatcher::getProxiesFromService(DictRef const& service, StringRef const& set_name)
 {
 	DictRef proxies(
 		(CFDictionaryRef)CFDictionaryGetValue(service, kSCEntNetProxies),
@@ -364,33 +365,38 @@ ProxyWatcher::getProxiesFromService(DictRef const& service)
 		throw AssertException(__FILE__, __LINE__);
 	}
 	
-	if (isProxyEnabled(proxies,  kSCPropNetProxiesHTTPEnable)) {
-		processHttpProxy(proxies);
-	} else if (isProxyEnabled(proxies, kSCPropNetProxiesSOCKSEnable)) {
-		processSocksProxy(proxies);
-	} else if (isProxyEnabled(proxies, CFSTR("ProxyAutoDiscoveryEnable"))) {
-		processProxyAutoDiscovery(proxies);
-	} else if (isProxyEnabled(proxies, CFSTR("ProxyAutoConfigEnable"))) {
-		processProxyAutoConfig(proxies);
+	Forwarding::SystemOption sys_opt(Forwarding::Utf8String(CFUtils::stringToUtf8(set_name)));
+	
+	if (isFlagSet(proxies,  kSCPropNetProxiesHTTPEnable)) {
+		processHttpProxy(proxies, sys_opt);
+	} else if (isFlagSet(proxies, kSCPropNetProxiesSOCKSEnable)) {
+		processSocksProxy(proxies, sys_opt);
+	} else if (isFlagSet(proxies, CFSTR("ProxyAutoDiscoveryEnable"))) {
+		processProxyAutoDiscovery(proxies, sys_opt);
+	} else if (isFlagSet(proxies, CFSTR("ProxyAutoConfigEnable"))) {
+		processProxyAutoConfig(proxies, sys_opt);
 	} else {
-		setNoProxy();
+		// No proxy. Nothing to be done with sys_opt.
 	}
+	
+	GlobalState::WriteAccessor()->forwardingResolver().selectSystemOption(sys_opt);
 }
 
 bool
-ProxyWatcher::isProxyEnabled(DictRef const& proxies, CFStringRef key)
+ProxyWatcher::isFlagSet(DictRef const& dict, CFStringRef key)
 {
-	CFNumberRef val = (CFNumberRef)CFDictionaryGetValue(proxies, key);
+	CFNumberRef val = (CFNumberRef)CFDictionaryGetValue(dict, key);
 	if (!val || CFGetTypeID(val) != CFNumberGetTypeID()) {
 		return false;
 	}
-	int enabled = 0;
-	CFNumberGetValue(val, kCFNumberIntType, &enabled);
-	return enabled != 0;
+	int flag = 0;
+	CFNumberGetValue(val, kCFNumberIntType, &flag);
+	return flag != 0;
 }
 
 void
-ProxyWatcher::processHttpProxy(DictRef const& proxies)
+ProxyWatcher::processHttpProxy(
+	DictRef const& proxies, Forwarding::SystemOption& sys_opt)
 {
 	CFStringRef host_str = (CFStringRef)CFDictionaryGetValue(
 		proxies, kSCPropNetProxiesHTTPProxy
@@ -418,20 +424,13 @@ ProxyWatcher::processHttpProxy(DictRef const& proxies)
 	proxy_desc.setType(ProxyDescriptor::HTTP);
 	proxy_desc.setAddr(SymbolicInetAddr(host, port));
 	
-	typedef std::list<std::string> SkipList;
-	SkipList skip_list;
-	DECLARE_INSERTER(SkipListInserter, std::string, SkipList, c.push_back(val));
-	SkipListInserter skip_list_inserter(skip_list);
-	readProxySkipList(proxies, skip_list_inserter);
-	
-	GlobalState::WriteAccessor global_state;
-	global_state->config().setNextHopProxy(proxy_desc);
-	global_state->config().setNextHopProxyEnabled(true);
-	global_state->config().setProxySkipList(skip_list);
+	sys_opt.proxyChain().push_back(proxy_desc);
+	readProxyBypassList(proxies, sys_opt);
 }
 
 void
-ProxyWatcher::processSocksProxy(DictRef const& proxies)
+ProxyWatcher::processSocksProxy(
+	DictRef const& proxies, Forwarding::SystemOption& sys_opt)
 {
 	CFStringRef host_str = (CFStringRef)CFDictionaryGetValue(
 		proxies, kSCPropNetProxiesSOCKSProxy
@@ -459,34 +458,36 @@ ProxyWatcher::processSocksProxy(DictRef const& proxies)
 	proxy_desc.setType(ProxyDescriptor::SOCKS5); // making a guess about SOCKS5
 	proxy_desc.setAddr(SymbolicInetAddr(host, port));
 	
-	typedef std::list<std::string> SkipList;
-	SkipList skip_list;
-	DECLARE_INSERTER(SkipListInserter, std::string, SkipList, c.push_back(val));
-	SkipListInserter skip_list_inserter(skip_list);
-	readProxySkipList(proxies, skip_list_inserter);
+	sys_opt.proxyChain().push_back(proxy_desc);
+	readProxyBypassList(proxies, sys_opt);
+}
+
+void
+ProxyWatcher::processProxyAutoDiscovery(
+	DictRef const& proxies, Forwarding::SystemOption& sys_opt)
+{
+	sys_opt.setErrorMessage("Unsupported feature: proxy auto discovery");
+}
+
+void
+ProxyWatcher::processProxyAutoConfig(
+	DictRef const& proxies, Forwarding::SystemOption& sys_opt)
+{
+	sys_opt.setErrorMessage("Unsupported feature: proxy auto configuration");
+}
+
+void
+ProxyWatcher::readProxyBypassList(
+	DictRef const& proxies, Forwarding::SystemOption& sys_opt)
+{
+	if (isFlagSet(proxies, CFSTR("ExcludeSimpleHostnames"))) {
+		sys_opt.bypassList().push_back(
+			Forwarding::BypassPatternConstPtr(
+				new Forwarding::SimpleHostnamesMatchPattern
+			)
+		);
+	}
 	
-	GlobalState::WriteAccessor global_state;
-	global_state->config().setNextHopProxy(proxy_desc);
-	global_state->config().setNextHopProxyEnabled(true);
-	global_state->config().setProxySkipList(skip_list);
-}
-
-void
-ProxyWatcher::processProxyAutoDiscovery(DictRef const& proxies)
-{
-	setGenericError("Unsupported feature: proxy auto discovery");
-}
-
-void
-ProxyWatcher::processProxyAutoConfig(DictRef const& proxies)
-{
-	setGenericError("Unsupported feature: proxy auto configuration");
-}
-
-void
-ProxyWatcher::readProxySkipList(
-	DictRef const& proxies, OutputFunction<std::string>& output)
-{
 	CFArrayRef arr = (CFArrayRef)CFDictionaryGetValue(
 		proxies, kSCPropNetProxiesExceptionsList
 	);
@@ -497,8 +498,6 @@ ProxyWatcher::readProxySkipList(
 		throw AssertException(__FILE__, __LINE__);
 	}
 	
-	char item_str[256];
-	
 	CFIndex const count = CFArrayGetCount(arr);
 	for (CFIndex i = 0; i < count; ++i) {
 		CFStringRef item = (CFStringRef)CFArrayGetValueAtIndex(arr, i);
@@ -506,37 +505,27 @@ ProxyWatcher::readProxySkipList(
 			throw AssertException(__FILE__, __LINE__);
 		}
 		
-		if (!CFStringGetCString(item, item_str, sizeof(item_str), kCFStringEncodingASCII)) {
-			// To support international domain names, we would have to convert
-			// strings to Punicode, which Core Foundation doesn't support.
-			continue;
-		}
-		
-		output(string(item_str));
-		output("."+string(item_str));
+		Forwarding::Utf8String const host_mask(CFUtils::stringToUtf8(item));
+		sys_opt.bypassList().push_back(
+			Forwarding::BypassPatternConstPtr(
+				new Forwarding::HostMaskMatchPattern(host_mask)
+			)
+		);
 	}
-	
-	// TODO: support ExcludeSimpleDomains property.
-	// This will require modifying the Config class.
 }
 
+/**
+ * Remember that SystemOption can be overriden by an option in forwarding.xml
+ * having the same name.  This function sets an error that can't be overriden.
+ */
 void
-ProxyWatcher::setNoProxy()
+ProxyWatcher::setUnmaskableError(std::string const& msg)
 {
-	ProxyDescriptor proxy;
-	GlobalState::WriteAccessor global_state;
-	global_state->config().setNextHopProxy(proxy);
-	global_state->config().setNextHopProxyEnabled(false);
-}
-
-void
-ProxyWatcher::setGenericError(std::string const& msg)
-{
-	ProxyDescriptor proxy;
-	proxy.setInvalid(msg);
-	GlobalState::WriteAccessor global_state;
-	global_state->config().setNextHopProxy(proxy);
-	global_state->config().setNextHopProxyEnabled(true);
+	// Options with empty names are not allowed, so nothing can override us.
+	Forwarding::Utf8String name;
+	Forwarding::SystemOption sys_opt(name);
+	sys_opt.setErrorMessage(msg);
+	GlobalState::WriteAccessor()->forwardingResolver().selectSystemOption(sys_opt);
 }
 
 
