@@ -38,6 +38,20 @@ function unpack
   tar "$flag" -xf "$fname"
 }
 
+# fetch 'URL'
+function fetch
+{
+  url="$1"
+  if which wget > /dev/null; then
+    wget "$url"
+  elif which curl > /dev/null; then
+    curl -O --retry 5 "$url"
+  else
+    echo "Neither wget nor curl is installed."
+    return 1
+  fi
+}
+
 # fetch_and_unpack 'URL'
 function fetch_and_unpack
 {
@@ -47,7 +61,7 @@ function fetch_and_unpack
     unpack "$fname"
   else
     echo "Fetching $url"
-    if wget -c "$url"; then
+    if fetch "$url"; then
       unpack "$fname"
     else
       rm -f "$fname"
@@ -62,17 +76,19 @@ function remove_subdirs
   file="$1"
   pattern="$2"
   script='
-/^[ \t]*SUBDIRS[ \t]*=/ {
-  :next
-  s/\<('$pattern')\>//g
-  /\\$/ {
-    n
-    b next
+  BEGIN { continuation = 0; }
+  $1 == "SUBDIRS" || continuation {
+    continuation = /\\$/;
+    for (i = 1; i <= NF; i++) {
+      if ($i ~ "^('$pattern')$") {
+        $i = "";
+      }
+    }
   }
-}
-'
-  sed -re "$script" "$file" > "$file.temp"
-  mv -fT "$file.temp" "$file"
+  { print }
+  '
+  awk "$script" "$file" > "$file.temp"
+  mv -f "$file.temp" "$file"
 }
 
 
